/********************************************************************
   TORNADO-PX5421Q DCM Demo Utility. rev.1A
   'C5421 DSP-to-DSP core communication test host loader for TORNADO-3x/E3x/P3x.
   (C) MicroLAB Systems, 2001


   Description:
   ------------
	This demo loads and controls C5421 DSP-to-DSP core communication via
	McBSP2 port. 

   Host TORNADO platform:
   ----------------------
	This demo utilize host TORNADO resources and runs in HOST mode.
	MIRAGE emulator or COFF-loader shall be used to upload
	code to host TORNADO board.

	TI CCS tools are required to recompile this demo and to
	upload code to host TORNADO on-board DSP environment.

  Copyright:
  ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
	  - DCM is installed onto TORNADO-3x/E3x/P3x mainboard
***********************************************************************/


#include <c3x_def.h>
#include <t3x_dsp.h>


// define C3x DSP platform and load PX5421Q host utilities
#define	_HOST_C3X_DSP_	1

#include <px5421qh.h>



// include ICC MASTER and SLAVE test code (C54x C-BSF files)
#include "master.cbs"
#include "slave.cbs"


init_cpu()
{
   *C3X_PBUS_GC=T3X_PBUS_GC_DATA;
}



#define	MASTER_PASS_COUNTER_HPI_ADDR		0x1800

unsigned pass_counter;


// returns master DSP pass counter value via HPI port
#define	master_pass_counter(n)	HTPX5421Q_rd_hpi_data(n, (((n%2)==0) ? HTPX5421Q_HPI_DARAMA_BADDR:HTPX5421Q_HPI_DARAMB_BADDR) + MASTER_PASS_COUNTER_HPI_ADDR)

// writes 0 to master DSP pass counter via HPI port
#define	init_master_pass_counter(n)	HTPX5421Q_wr_hpi_data(n, (((n%2)==0) ? HTPX5421Q_HPI_DARAMA_BADDR:HTPX5421Q_HPI_DARAMB_BADDR) + MASTER_PASS_COUNTER_HPI_ADDR, 0)



void	_pause()	
{ int i;
	for(i=0;i<10000;i++);
}


// define ICC DSP core test array, which contains
// DSP cores IDs, which are involved into the test.
//			McBSP2@DSP0A <--> McBSP2@DSP1A
//			McBSP2@DSP0B <--> McBSP2@DSP2B
//			McBSP2@DSP2A <--> McBSP2@DSP3A
//			McBSP2@DSP1B <--> McBSP2@DSP3B

typedef struct
{
	unsigned master, slave;
} DSP_CORES_PAIR_DD;


DSP_CORES_PAIR_DD	dsp_test_cores[4]= { {HTPX5421Q_DSP0A,HTPX5421Q_DSP1A},
										 {HTPX5421Q_DSP0B,HTPX5421Q_DSP2B},
										 {HTPX5421Q_DSP2A,HTPX5421Q_DSP3A},
										 {HTPX5421Q_DSP1B,HTPX5421Q_DSP3B}
									   };
					



main()
{  int n, err;
   unsigned long timeout;

   init_cpu();
	
	err = 10;

	// set DSP RESET state for all DSP cores and release HPI RESET
	if(HTPX5421Q_init()!=HTPX5421Q_OK) 
		goto error;

    HTPX5421Q_SET_HPI_GO;

	for(n=0;n<4;n++)
	{
		// set DSP reset for all cores and load MASTER and SLAVE code to the
		// DSP cores under the test.
		HTPX5421Q_SET_DSP_MRESET_ALL;

		err = 20;
		if( 	HTPX5421Q_load_program(dsp_test_cores[n].master,C54X_CBSF_DATA_MASTER)!=HTPX5421Q_OK
			||	HTPX5421Q_load_program(dsp_test_cores[n].slave,C54X_CBSF_DATA_SLAVE)!=HTPX5421Q_OK
		  )
				goto error;

		init_master_pass_counter(dsp_test_cores[n].master);

		// now release DSP reset for SLAVE and MASTER (separated by the pause)
		HTPX5421Q_SET_DSP_MGO(dsp_test_cores[n].slave);
		_pause();
		HTPX5421Q_SET_DSP_MGO(dsp_test_cores[n].master);


		// now wait until MASTER PASS COUNTER will come equeal to 100
		// perform timeout control (1ms) via background loop
		timeout = 0L;

		while(1)
		{
			// test for pass counter
			if ( master_pass_counter(dsp_test_cores[n].master) >=100)
				break;

			// test for timeout
			if((timeout++) > 0xffff) goto error;
		}
	}


	//---------------- execution trap code ------------

  exit_ok:	
	// get here on successful execution
	// set HPI_RESET to GO and all DSP RESETs to the RUN state for visual indication
   HTPX5421Q_SET_DSP_MRESET_ALL;
   HTPX5421Q_SET_HPI_GO;
   HTPX5421Q_SET_DSP_MGO_ALL;
   while(1);


  error:	
	// get here on any error occured
	// The following visual indication is provided:
	//	- set HPI_RESET to RESET and DSP reset to the 01010101 bin pattern in case
	//    error code err<20 (general init errors)
	//	- set HPI_RESET to RESET and DSP RESETs to the OFF state for those
	//	  DSP cores, which have passed test successfully in case err>=20
   HTPX5421Q_SET_DSP_MRESET_ALL;
   HTPX5421Q_SET_HPI_RESET;

		// analyse error tracking code (err)
	if(err<20)
		// this was incorrect general init, load '01010101' pattern
	   HTPX5421Q_SET_CNTR1_RG(0x55);
	else
	{
		// general initialization has passed successfully
		// Release DSP RESET LEDs for those DSP cores, which have passed
		// test successfully.
		if(n>0)
		{
			HTPX5421Q_SET_DSP0A_MGO;
			if(n>1)
			{
				HTPX5421Q_SET_DSP0B_MGO;
				if(n>2)
				{
					HTPX5421Q_SET_DSP1A_MGO;
					if(n>3)
					{
						HTPX5421Q_SET_DSP1B_MGO;
					}
				}
			}
		}
	}

		// loop here infinitely
	error_loop:	
	    while(1) ;


}


