/****************************************************
  TORNADO-PX5421 DSP combined SIOX test. v.1
  (C) MicroLAB Systems, 2001

  note:
    1. connect external pulse generator to XINT-0/1 inputs (10kHz), 
	   which esults in:
        - XIO-0 output toggles on each falling edge of XINT0
        - XIO-1 output toggles on each falling edge of XINT1
	2. McBSP-0/1 DX/FSX/FSR/CLKX/CLKR pins are configured as OUTPUTs.
	   External DX=DR plug-in jumpers are required at each of the 
       McBSP-0/1 port of mini-SIOX extender. Output pulses at the DX/DR, 
	   CLKX, CLKR, FSR, FSX outputs for McBSP-0/1 port appear in case of 
       correct operation. Use oscilloscope in order to observe these pulses.
	4. SIOX RESET output appears as the short pulse. Use oscilloscope in
	   order to observe these pulses.

*****************************************************/

#include "c54x_def.h"
#include "px5421qd.h"


// interrupt handler for XINT-0
// toggles XIO-0 output on each XINT-0 event
interrupt void c_xint0()
{
  TPX5421Q_set_xio0(~TPX5421Q_xio0_data());
}

// interrupt handler for XINT-1
// toggles XIO-1 output on each XINT-1 event
interrupt void c_xint1()
{
  TPX5421Q_set_xio1(~TPX5421Q_xio1_data());
}


//McBSP output pns defs
#define DX_OUT_0      0x3f00
#define DX_OUT_1      0x3f20
#define FSX_OUT_0     0x3f00
#define FSX_OUT_1     0x3f08
#define FSR_OUT_0     0x3f00
#define FSR_OUT_1     0x3f04
#define CLKX_OUT_0    0x3f00
#define CLKX_OUT_1    0x3f02
#define CLKR_OUT_0    0x3f00
#define CLKR_OUT_1    0x3f01

#define DR_BIT_MASK   0x10
#define CLKS_BIT_MASK 0x40
#define CLKR_BIT_MASK 0x1
#define CLKX_BIT_MASK 0x2
#define FSX_BIT_MASK  0x8
#define FSR_BIT_MASK  0x4


// some time delay is required betwen setting DX and testing 
// returned DR due to the extender cable delays
#define _pause()	asm(" nop")



main()
{
   TPX5421Q_init_dsp();

	// configure XIO-0/1 pins as ouputs
   TPX5421Q_set_xio0_cnf(TPX5421Q_XIO_CNF_OUTPUT);
   TPX5421Q_set_xio0(OFF);
   TPX5421Q_set_xio1_cnf(TPX5421Q_XIO_CNF_OUTPUT);
   TPX5421Q_set_xio1(OFF);

   asm(" stm #0h,IFR");        // clear IFR
   asm(" stm #3h,IMR");        // enable IRQ-0/1

   C54X_INT_ENABLE;			   // enable DSP interrupts

	// configure McBSP port pins as I/O pins
   *C54X_MCBSP0_SPSA = C54X_MCBSP_SBA_SPCR1;
   *C54X_MCBSP0_SPCR1 = 0;  

   *C54X_MCBSP0_SPSA = C54X_MCBSP_SBA_SPCR2;
   *C54X_MCBSP0_SPCR2 = 0;


   *C54X_MCBSP1_SPSA = C54X_MCBSP_SBA_SPCR1;
   *C54X_MCBSP1_SPCR1 = 0;   

   *C54X_MCBSP1_SPSA = C54X_MCBSP_SBA_SPCR2;
   *C54X_MCBSP1_SPCR2 = 0;

	// set SPSA registers pointing to the PCR reg
   *C54X_MCBSP0_SPSA = C54X_MCBSP_SBA_PCR;
   *C54X_MCBSP1_SPSA = C54X_MCBSP_SBA_PCR;


   while(1)
    {	
		// toggle SX RESET output
       TPX5421Q_set_sx_run(TPX5421Q_ON);
      if(TPX5421Q_sx_run_data()!=TPX5421Q_ON) goto error;

       TPX5421Q_set_sx_run(TPX5421Q_OFF);
      if(TPX5421Q_sx_run_data()!=TPX5421Q_OFF) goto error;


      // generate McBSP-0 FSX/FSR/CLKX/CLKR/DX-DR output pulses
	  // test for correct DX output return via DR input
      *C54X_MCBSP0_PCR = FSX_OUT_1;
      *C54X_MCBSP0_PCR = FSX_OUT_0;

      *C54X_MCBSP0_PCR = FSR_OUT_1;
      *C54X_MCBSP0_PCR = FSR_OUT_0;

      *C54X_MCBSP0_PCR = CLKX_OUT_1;
      *C54X_MCBSP0_PCR = CLKX_OUT_0;

      *C54X_MCBSP0_PCR = CLKR_OUT_1;
      *C54X_MCBSP0_PCR = CLKR_OUT_0;

      *C54X_MCBSP0_PCR = DX_OUT_1;
	  _pause();
      if(((*C54X_MCBSP0_PCR) & DR_BIT_MASK  ) ==0) break; 

      *C54X_MCBSP0_PCR = DX_OUT_0;
	  _pause();
      if(((*C54X_MCBSP0_PCR) & DR_BIT_MASK  ) !=0) break; 



      // generate McBSP-1 FSX/FSR/CLKX/CLKR/DX-DR output pulses
	  // test for correct DX output return via DR input
      *C54X_MCBSP1_PCR = FSX_OUT_1;
      *C54X_MCBSP1_PCR = FSX_OUT_0;

      *C54X_MCBSP1_PCR = FSR_OUT_1;
      *C54X_MCBSP1_PCR = FSR_OUT_0;

      *C54X_MCBSP1_PCR = CLKX_OUT_1;
      *C54X_MCBSP1_PCR = CLKX_OUT_0;

      *C54X_MCBSP1_PCR = CLKR_OUT_1;
      *C54X_MCBSP1_PCR = CLKR_OUT_0;

      *C54X_MCBSP1_PCR = DX_OUT_1;
	  _pause();
      if(((*C54X_MCBSP1_PCR) & DR_BIT_MASK  ) ==0) break; 

      *C54X_MCBSP1_PCR = DX_OUT_0;
	  _pause();
      if(((*C54X_MCBSP1_PCR) & DR_BIT_MASK  ) !=0) break; 
    }

	// get here on any error
	error:
    	while(1);

}


