/********************************************************************
   TORNADO-PX5421Q DCM Demo Utility. rev.1A
   Host-to-DSP communication test for T-PX5421Q and TORNADO-3x/E3x/P3x.
   (C) MicroLAB Systems, 2001


   file:
   -----
   	Host TORNADO-3x/E3x/P3x driver.

   Description:
   ------------
	This demo performs communication test between DSP of host
	TORNADO-3x/E3x/P3x mainboard and each DSP of T-PX5421Q DCM:
	 - test for communication via HPI ports
	 - test for HPI-port Interrupt communication

   Host TORNADO platform:
   ----------------------
	This demo utilize host TORNADO resources and runs in HOST mode.
	MIRAGE emulator or COFF-loader shall be used to upload
	code to host TORNADO board.

	This demo utilize T-PX5421Q on-board DSP resources. MIRAGE
	emulator is required to upload DSP code to each DSP core.

	TI CCS tools are required to recompile this demo.

  Copyright:
  ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
	  - DCM is installed onto TORNADO-3x/E3x/P3x mainboard
***********************************************************************/


#include <c3x_def.h>
#include <t3x_dsp.h>


// define C3x DSP platform and load PX5421Q host utilities
#define	_HOST_C3X_DSP_	1

#include <px5421qh.h>


// include C5421 DSP core code in C-BSF file
#include "px54_mc.cbs"


init_cpu()
{
   *C3X_PBUS_GC=T3X_PBUS_GC_DATA;
}



unsigned sa;
unsigned *D,data,data_start, data_rd;


#define	MCD_ADDR				0x4000	// MCD address in DSP on-chip DARAM0

#define	M_ID					0x110
#define	PRG_ID					0x111

#define	MCMND_NOP				0
#define	MCMND_HPI_COMM			1
#define	MCMND_IRQ_COMM			2
#define	MCMND_HPI_HINT_COMM		3

#define	MSTAT_BUSY				2
#define	MSTAT_ATTN				1
#define	MSTAT_RDY				0

#define	ERR_OK					0
#define	ERR_INV_CMND			-2
#define	ERR_INV_PARAM			-1


// components addresses fpr host-to-DSP communication descriptor

#define	MCD_MID_ADDR			MCD_ADDR
#define	MCD_PRG_ID_ADDR			(MCD_ADDR+1)
#define	MCD_CMND_ADDR			(MCD_ADDR+2)
#define	MCD_STATUS_ADDR			(MCD_ADDR+3)
#define	MCD_ERROR_ADDR			(MCD_ADDR+4)
#define	MCD_UP0_ADDR			(MCD_ADDR+5)
#define	MCD_UP1_ADDR			(MCD_ADDR+6)
#define	MCD_UP2_ADDR			(MCD_ADDR+7)
#define	MCD_UP3_ADDR			(MCD_ADDR+8)

#define	_HPI_DATA_ADDR(n,a)		(a+((n%2)==0? 0:HTPX5421Q_HPI_DARAMB_BADDR))

#define	SET_MCD_MID(j,d)		HTPX5421Q_wr_hpi_data(j,_HPI_DATA_ADDR(j,MCD_MID_ADDR), d)
#define	SET_MCD_PRG_ID(j,d)		HTPX5421Q_wr_hpi_data(j,_HPI_DATA_ADDR(j,MCD_PRG_ID_ADDR), d)
#define	SET_MCD_CMND(j,d)		HTPX5421Q_wr_hpi_data(j,_HPI_DATA_ADDR(j,MCD_CMND_ADDR), d)
#define	SET_MCD_STATUS(j,d)		HTPX5421Q_wr_hpi_data(j,_HPI_DATA_ADDR(j,MCD_STATUS_ADDR), d)
#define	SET_MCD_ERROR(j,d)		HTPX5421Q_wr_hpi_data(j,_HPI_DATA_ADDR(j,MCD_ERROR_ADDR), d)
#define	SET_MCD_UP0(j,d)		HTPX5421Q_wr_hpi_data(j,_HPI_DATA_ADDR(j,MCD_UP0_ADDR), d)
#define	SET_MCD_UP1(j,d)		HTPX5421Q_wr_hpi_data(j,_HPI_DATA_ADDR(j,MCD_UP1_ADDR), d)
#define	SET_MCD_UP2(j,d)		HTPX5421Q_wr_hpi_data(j,_HPI_DATA_ADDR(j,MCD_UP2_ADDR), d)
#define	SET_MCD_UP3(j,d)		HTPX5421Q_wr_hpi_data(j,_HPI_DATA_ADDR(j,MCD_UP3_ADDR), d)

#define	MCD_MID(j)				HTPX5421Q_rd_hpi_data(j,_HPI_DATA_ADDR(j,MCD_MID_ADDR))
#define	MCD_PRG_ID(j)			HTPX5421Q_rd_hpi_data(j,_HPI_DATA_ADDR(j,MCD_PRG_ID_ADDR))
#define	MCD_CMND(j)				HTPX5421Q_rd_hpi_data(j,_HPI_DATA_ADDR(j,MCD_CMND_ADDR))
#define	MCD_STATUS(j)			HTPX5421Q_rd_hpi_data(j,_HPI_DATA_ADDR(j,MCD_STATUS_ADDR))
#define	MCD_ERROR(j)			HTPX5421Q_rd_hpi_data(j,_HPI_DATA_ADDR(j,MCD_ERROR_ADDR))
#define	MCD_UP0(j)				HTPX5421Q_rd_hpi_data(j,_HPI_DATA_ADDR(j,MCD_UP0_ADDR))
#define	MCD_UP1(j)				HTPX5421Q_rd_hpi_data(j,_HPI_DATA_ADDR(j,MCD_UP1_ADDR))
#define	MCD_UP2(j)				HTPX5421Q_rd_hpi_data(j,_HPI_DATA_ADDR(j,MCD_UP2_ADDR))
#define	MCD_UP3(j)				HTPX5421Q_rd_hpi_data(j,_HPI_DATA_ADDR(j,MCD_UP3_ADDR))



//---- these functions return stutus of the IRQ flags for XIRQ-0/1/2
int _xirq0_data()
{
   asm(" ldi if,r0");
   asm(" and 1,r0");
}

int _xirq1_data()
{
   asm(" ldi if,r0");
   asm(" and 2,r0");
}

int _xirq2_data()
{
   asm(" ldi if,r0");
   asm(" and 4,r0");
}



int	_xirq_data(n)
{
	switch(n)
	{
		case 0:
			return (_xirq0_data());

		case 1:
			return (_xirq1_data());

		default:
			return (_xirq2_data());
	}
}



int n=0;	// is used for tracking error conditions


main()
{ unsigned d,i,j,k,kk;


   init_cpu();


   n=10;

	// set DSP RESET state for all DSP cores, dsable all HIRQs
   if(HTPX5421Q_init()!=HTPX5421Q_OK) goto error;

   n=11;

	// release HPI RESET
   HTPX5421Q_SET_HPI_GO;
   HTPX5421Q_SET_HPI_ERR_EN_ON;
   if(HTPX5421Q_CNTR2_RG_DATA!= (HTPX5421Q_HPI_ERR_EN_ON | HTPX5421Q_HPI_RESET_GO))
		goto error;


	n=12;
   // load DSP core code fm the C-BSF array
   for(j=0;j<8;j++)
	{
		if(HTPX5421Q_load_program(j,C54X_CBSF_DATA)!=HTPX5421Q_OK)
			goto error;
	}


	// perform test for each DSP core
   for(j=0;j<8;j++)
	{
	   n=20;

		// disable all host IRQs
	   HTPX5421Q_SET_HIRQ_EN_OFF_ALL;
	   HTPX5421Q_SET_HIRQ0_IE1_RG(HTPX5421Q_HINT_IE_OFF_ALL);
	   HTPX5421Q_SET_HIRQ1_IE1_RG(HTPX5421Q_HINT_IE_OFF_ALL);
	   HTPX5421Q_SET_HIRQ2_IE1_RG(HTPX5421Q_HINT_IE_OFF_ALL);
	   HTPX5421Q_SET_HIRQ3_IE1_RG(HTPX5421Q_HINT_IE_OFF_ALL);
	   HTPX5421Q_SET_HIRQ0_IE2_RG(HTPX5421Q_HPI_ERR_IE_OFF);
	   HTPX5421Q_SET_HIRQ1_IE2_RG(HTPX5421Q_HPI_ERR_IE_OFF);
	   HTPX5421Q_SET_HIRQ2_IE2_RG(HTPX5421Q_HPI_ERR_IE_OFF);
	   HTPX5421Q_SET_HIRQ3_IE2_RG(HTPX5421Q_HPI_ERR_IE_OFF);

	   if(   HTPX5421Q_HIRQ_EN_RG_DATA!=0
		  || HTPX5421Q_HIRQ0_IE1_RG_DATA!=0
		  || HTPX5421Q_HIRQ1_IE1_RG_DATA!=0
		  || HTPX5421Q_HIRQ2_IE1_RG_DATA!=0
		  || HTPX5421Q_HIRQ3_IE1_RG_DATA!=0
		  || HTPX5421Q_HIRQ0_IE2_RG_DATA!=0
		  || HTPX5421Q_HIRQ1_IE2_RG_DATA!=0
		  || HTPX5421Q_HIRQ2_IE2_RG_DATA!=0
		  || HTPX5421Q_HIRQ3_IE2_RG_DATA!=0
		 ) 
			goto error;

	   n=21;

		// clear XIRQs and verify the result (PIOX at T3x provides XIRQ-0..2 only)
	   	C3X_XINT_CLEAR(0);
   		C3X_XINT_CLEAR(1);
		C3X_XINT_CLEAR(2);

	   if(   _xirq0_data()!=0
    	  || _xirq1_data()!=0
      	  || _xirq2_data()!=0
     	 ) 
			goto error;

	   n=22;

	   	// initialize MCD, release DSP reset and test returned DSP reply
	   SET_MCD_MID(j,0xffff);
	   SET_MCD_PRG_ID(j,0xffff);
	   SET_MCD_CMND(j,0xffff);
	   SET_MCD_STATUS(j, 0xffff);
	   SET_MCD_ERROR(j,0x7fff);
	   SET_MCD_UP0(j,0xffff);
	   SET_MCD_UP1(j,0xffff);

	   HTPX5421Q_SET_DSP_MGO(j);


		// wait for DSP will init
   	   while (MCD_STATUS(j)!=MSTAT_RDY)
	     {
      		if(HTPX5421Q_HPI_ERR_DATA(n/2)!=0) goto error;
    	 }

	   n=23;

		// validate MCD returned
   		if(	   MCD_MID(j)!=M_ID      	
      		|| MCD_PRG_ID(j)!=PRG_ID 	
      		|| MCD_CMND(j)!=MCMND_NOP
      		|| MCD_STATUS(j)!=MSTAT_RDY
      		|| MCD_ERROR(j)!=ERR_OK
      		|| MCD_UP0(j)!=0x5555
      		|| MCD_UP1(j)!=0xaaaa
     	 )
		     goto error;




	   n=24;

		//------- testing communication via HPI ...

	   SET_MCD_CMND(j,MCMND_HPI_COMM);
	   SET_MCD_ERROR(j,0x7fed);
	   SET_MCD_UP0(j,0xffff);
	   SET_MCD_UP1(j,0xffff);
	   SET_MCD_UP2(j,0xffff);
	   SET_MCD_UP3(j,0xffff);
	   SET_MCD_STATUS(j,MSTAT_ATTN);


		// wait for DSP will reply
   	   while (MCD_STATUS(j)!=MSTAT_RDY)
	     {
      		if(HTPX5421Q_HPI_ERR_DATA(n/2)!=0) goto error;
    	 }

	   n=30;


  	   if(     MCD_ERROR(j)!=ERR_OK
      		|| MCD_UP0(j)!=0xfedc
      		|| MCD_UP1(j)!=0xba98
      		|| MCD_UP2(j)!=0x7654
      		|| MCD_UP3(j)!=0x3210
     	 )
	       goto error;

	   n=31;


		//---- testing communication via DSPINT ...
	   SET_MCD_CMND(j,MCMND_IRQ_COMM);
	   SET_MCD_ERROR(j,0xffff);
	   SET_MCD_UP0(j,0xffff);
	   SET_MCD_UP1(j,0xffff);
	   SET_MCD_UP2(j,0xffff);
	   SET_MCD_UP3(j,0x5143);

	   HTPX5421Q_SET_HPIC_DSPINT(j);
	   

		// wait for DSP will reply
   	   while (MCD_ERROR(j)!=ERR_OK)
	     {
      		if(HTPX5421Q_HPI_ERR_DATA(n/2)!=0) goto error;
    	 }

	   n=40;


  	   if(     MCD_UP0(j)!=0x0123
      		|| MCD_UP1(j)!=0x4567
      		|| MCD_UP2(j)!=0x89ab
      		|| MCD_UP3(j)!=((~0x5143) & 0xffff)
     	 )
	       goto error;




	   	//------ testing HPI/HINT for XIRQ0..2
		// (PIOX at T3x/E3x provides XIRQ-0..2 only)
		for(k=0;k<3;k++)
		{		
			n=50;
	  		if(HTPX5421Q_HPI_ERR_DATA(n/2)!=0) goto error;
 		    SET_MCD_STATUS(j,MSTAT_RDY);


				//--- enable XIRQ-k and set HINT_EI(j) bit
		    HTPX5421Q_SET_HIRQ_EN_ON(k);	
	   		HTPX5421Q_SET_HIRQ_IE_ON(k,j);

			n=51;
		   if(   _xirq_data(k)!=0
    		  || HTPX5421Q_HIRQ_IE_DATA(k,j)==0
      		  || HTPX5421Q_HINT_DATA(j)!=0
	      	  || HTPX5421Q_HIRQ_EN_DATA(k)==0
		     ) 
				goto error;


		   SET_MCD_CMND(j,MCMND_HPI_HINT_COMM);
		   SET_MCD_ERROR(j,0x7fed);
		   SET_MCD_STATUS(j,MSTAT_ATTN);

			n=52;
			// wait for DSP will reply
   		   while (MCD_STATUS(j)!=MSTAT_RDY)
	    	 {
      			if(HTPX5421Q_HPI_ERR_DATA(n/2)!=0) goto error;
	    	 }

			n=53;

	  	   if(   	MCD_ERROR(j)!=ERR_OK
				||  HTPX5421Q_HINT_DATA(j)==0
			  )
		       goto error;

			n=54;

			// test for correct XIRQ has been generated
		   for(kk=0;kk<3;kk++)
			{
			   if(   (kk!=k  && _xirq_data(kk)!=0)
				  || (kk==k  && _xirq_data(kk)==0)
				 )
		           	goto  error;
			}

			n=55;

			// clear XINT IRQ
		   switch(k)
			{
				case 0:
					C3X_XINT_CLEAR(0);
					break;

				case 1:
					C3X_XINT_CLEAR(1);
					break;

				default:
					C3X_XINT_CLEAR(2);
					break;
			}

		   HTPX5421Q_CLR_HPIC_HINT(j);
   		   if(HTPX5421Q_HPIC_HINT_DATA(j)!=0) 
			  goto error;

		   HTPX5421Q_SET_HIRQ_IE_OFF(k,j);
		   HTPX5421Q_SET_HIRQ_EN_OFF(k);
		}

	}


	//---------------- execution trap code ------------

  exit_ok:	
	// get here on successful execution
	// set HPI_RESET to GO and all DSP RESETs to the RUN state for visual indication
   HTPX5421Q_SET_DSP_MRESET_ALL;
   HTPX5421Q_SET_HPI_GO;
   HTPX5421Q_SET_DSP_MGO_ALL;
   while(1);


  error:	
	// get here on any error occured
	// The following visual indication is provided:
	//	- set HPI_RESET to RESET and DSP reset to the 01010101 bin pattern in case
	//    error code N<20 (general init errors)
	//	- set HPI_RESET to RESET and DSP RESETs to the OFF state for those
	//	  DSP cores, which have passed test successfully in case N>=20

   HTPX5421Q_SET_DSP_MRESET_ALL;
   HTPX5421Q_SET_HPI_RESET;

		// analyse error tracking code (N)
	if(n<20)
		// this was incorrect general init, load '01010101' pattern
	   HTPX5421Q_SET_CNTR1_RG(0x55);
	else
	{
		// general initialization has passed successfully
		// Release DSP RESET LEDs for those DSP cores, which have passed
		// test successfully.
		if(j>0)
		{
			HTPX5421Q_SET_DSP0A_MGO;
			if(j>1)
			{
				HTPX5421Q_SET_DSP0B_MGO;
				if(j>2)
				{
					HTPX5421Q_SET_DSP1A_MGO;
					if(j>3)
					{
						HTPX5421Q_SET_DSP1B_MGO;
						if(j>4)
						{
							HTPX5421Q_SET_DSP2A_MGO;
							if(j>5)
							{
								HTPX5421Q_SET_DSP2B_MGO;
								if(j>6)
								{
									HTPX5421Q_SET_DSP3A_MGO;
									if(j>7)
										HTPX5421Q_SET_DSP3B_MGO;
								}
							}
						}
					}
				}
			}
		}
	}

		// loop here infinitely
	error_loop:	
	    while(1) ;

}


