/********************************************************************
   TORNADO-PX5421Q DCM Demo Utility. rev.1A
   Host-to-DSPcore communication test for PX5421Q and TORNADO-3x/E3x/P3x.
   (C) MicroLAB Systems, 2001

   file:
   -----
   	DSP core resident driver for TORNADO-PX5421Q4 DCM.

   Description:
   ------------
	This is the T-PX5421Q resident DSP component of complex
	communication test between DSP of host TORNADO-3x/E3x/P3x
	mainboard and DSP of T-PX5421Q DCM. Refer for details to the
	host driver.

   Host TORNADO platform:
   ----------------------
	This demo utilize host TORNADO resources and runs in HOST mode.
	MIRAGE emulator or COFF-loader shall be used to upload
	code to host TORNADO board.

	This demo utilize T-PX5421Q on-board DSP core resources. MIRAGE
	emulator is required to upload code to the DSP of T-PX5421Q.

	TI CCS tools are required to recompile this demo.

  Copyright:
  ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
	  - DCM is installed onto TORNADO-3x/E3x/P3x mainboard
***********************************************************************/


#include "c54x_def.h"
#include "px5421qd.h"



#define	M_ID					0x110
#define	PRG_ID					0x111

#define	MCMND_NOP				0
#define	MCMND_HPI_COMM			1
#define	MCMND_IRQ_COMM			2
#define	MCMND_HPI_HINT_COMM		3

#define	MSTAT_BUSY				2
#define	MSTAT_ATTN				1
#define	MSTAT_RDY				0

#define	ERR_OK					0
#define	ERR_INV_CMND			-2
#define	ERR_INV_PARAM			-1
#define	ERR_INV_INIT			-100
#define	ERR_CMND_EXEC_ERR		-3


// allocation address for host-to-DSP communication descriptor
#define	MCD_ADDR				0x4000	


// host-to-DSP communication descriptor.
struct MCD_DD
{ int  	m_id,
		prg_id,
		cmnd,
		status,
		error;

   unsigned     up[4];
};

struct MCD_DD *MCD;


// ISR on HPI IRQ event
interrupt void c_dspint_isr()
{ 
  if(MCD->cmnd==MCMND_IRQ_COMM)
	{  MCD->up[0]=0x0123;
	   MCD->up[1]=0x4567;
	   MCD->up[2]=0x89ab;
	   MCD->up[3]= ~MCD->up[3];
	   MCD->error=ERR_OK;
	}
}




main()
{int d,i;

	TPX5421Q_init_dsp();


    // enable DSPINT
   C54X_IFR_CLEAR;        
   *C54X_IMR=0;
   C54X_HPI_INT_ENABLE;     
   C54X_INT_ENABLE;


	// initialize MCD data
	MCD=(struct MCD_DD *) MCD_ADDR;

    MCD->m_id=M_ID;
    MCD->prg_id=PRG_ID;
    MCD->cmnd=MCMND_NOP;
	MCD->error=ERR_OK;
	MCD->up[0]=0x5555;
	MCD->up[1]=0xaaaa;
	MCD->status=MSTAT_RDY;


   while(1)
    {
		// wait for software host-to-DSP request
       while(MCD->status !=MSTAT_ATTN);
       MCD->status=MSTAT_BUSY;

       switch(MCD->cmnd)
		{
          case MCMND_NOP:
	       MCD->error=ERR_OK;
	       break;

          case MCMND_HPI_COMM:
	        MCD->up[0]=0xfedc;
			MCD->up[1]=0xba98;
			MCD->up[2]=0x7654;
			MCD->up[3]=0x3210;
	        MCD->error=ERR_OK;
	        break;

          case MCMND_HPI_HINT_COMM:
		  	C542X_SET_HPIC_HINT;
 	        MCD->error=ERR_OK;
	        break;


          default:
	        MCD->error=ERR_INV_CMND;
	        break;

	}

       MCD->status=MSTAT_RDY;
    }


   error:
      while(1);

}



