/********************************************************************
   TORNADO-PX5421Q DCM Demo Utility. rev.1A
   'C5421 DSP HPI test for TORNADO-3x/E3x/P3x.
   (C) MicroLAB Systems, 2001


   Description:
   ------------
	This demo performs HPI test for each DSP core of TORNADO-PX5421Q 
	via host TORNADO-3x/E3x/P3x mainboard. 

   Host TORNADO platform:
   ----------------------
	This demo utilize host TORNADO resources and runs in HOST mode.
	MIRAGE emulator or COFF-loader shall be used to upload
	code to host TORNADO board.

	TI CCS tools are required to recompile this demo and to
	upload code to host TORNADO on-board DSP environment.

  Copyright:
  ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
	  - DCM is installed onto TORNADO-3x/E3x/P3x mainboard
***********************************************************************/


#include <c3x_def.h>
#include <t3x_dsp.h>


// define C3x DSP platform and load PX5421Q host utilities
#define	_HOST_C3X_DSP_	1

#include <px5421qh.h>



init_cpu()
{
   *C3X_PBUS_GC=T3X_PBUS_GC_DATA;
}



unsigned 	 data_inc_da     	= 0xb5f1,
			 data_inc_db     	= 0x79d3,
			 data_inc_p01      	= 0xa359,
			 data_inc_p23      	= 0xb947,

			 initial_data_da 	= 0xe97f,
			 initial_data_p01 	= 0x7531,
			 initial_data_db 	= 0x95db,
			 initial_data_p23 	= 0xfa95,

		     data_start_da,
		     data_start_p01,
		     data_start_db, 
		     data_start_p23;

unsigned pass_counter;


int fill_hpi_ram(unsigned n, unsigned saddr,unsigned eaddr,unsigned sdata)
// fill in the DSP#N on-chip memory via HPI	
{ unsigned sa,data;

  for (sa=saddr, data=sdata ;sa<=eaddr;sa++)
  {
	  HTPX5421Q_SET_HPI_ADDR(n,sa);
	  if(HTPX5421Q_HPI_ADDR_DATA(n)!=sa) return(-1);

  	  HTPX5421Q_SET_HPID(n,data);
      if(HTPX5421Q_HPI_ERR_DATA(n/2)!=0) return(-1);

	  data++;
	  data &= 0xffff;
  }

  return(0);
}


int verify_hpi_ram(unsigned n,unsigned saddr,unsigned eaddr, unsigned sdata)
// verify the DSP#N on-chip memory via HPI	
{ unsigned sa,data, data_rd;

  	//-- verify HPI data using address auto increment mode
  data = sdata;

  HTPX5421Q_SET_HPI_ADDR(n,saddr);
  if(HTPX5421Q_HPI_ADDR_DATA(n)!=saddr) return(-1);

  for (sa=saddr, data=sdata ;sa<=eaddr;sa++)
  {
	data_rd=HTPX5421Q_HPID_AINC_DATA(n);
    if(HTPX5421Q_HPI_ERR_DATA(n/2)!=0)  return(-1);

    if(data_rd!=data)  return(-1);

	data++;
	data &= 0xffff;
  }

  return(0);
}


void	_pause()	
{ int i;
	for(i=0;i<100;i++);
}



main()
{ 	int i,n;
  	unsigned d;
	unsigned data_start_da_start, 
			 data_start_p01_start, 
			 data_start_db_start,
			 data_start_p23_start;


   init_cpu();
	

	// set DSP RESET state for all DSP cores and release HPI RESET
	if(HTPX5421Q_init()!=HTPX5421Q_OK) 
		goto error;

   HTPX5421Q_SET_HPI_GO;
   if(   HTPX5421Q_CNTR1_RG_DATA!=HTPX5421Q_DSP_MRESET_RESET_ALL
	  || HTPX5421Q_CNTR2_RG_DATA!= (HTPX5421Q_HPI_ERR_EN_ON | HTPX5421Q_HPI_RESET_GO)
	  || HTPX5421Q_HPI_ERR_STAT_RG_DATA!=0
	 ) goto error;


	   data_start_da_start = initial_data_da;
	   data_start_p01_start = initial_data_p01;
	   data_start_db_start = initial_data_db;
	   data_start_p23_start = initial_data_p23;

   	   pass_counter =0;


		// provide HPI memory scan
	 while(1)
     {


		// assume that all DSPs are in RESET state and perform DSP on-chip memory test
	    if(   HTPX5421Q_CNTR1_RG_DATA!=HTPX5421Q_DSP_MRESET_RESET_ALL
		   || HTPX5421Q_HPI_ERR_STAT_RG_DATA!=0
	 	  ) 
			goto error;

	   data_start_da = data_start_da_start;
	   data_start_p01 = data_start_p01_start;
	   data_start_db = data_start_db_start;
	   data_start_p23 = data_start_p23_start;


		// write DSP on-chip D/P-memory for each DSP chip (not core!!!)
		for(n=0;n<8;n+=2)
		{
			// initilize C54x HPI port 
	   		HTPX5421Q_HPI_INIT(n);

		   if(		// first fill-in DARAM/SARAM data memory for DSP-A core
				    fill_hpi_ram(n,HTPX5421Q_HPI_DARAMA_SADDR,HTPX5421Q_HPI_SARAMA_EADDR,data_start_da) !=0

					// now fill-in the SPDARAM0/1 shared program memory
				||  fill_hpi_ram(n,HTPX5421Q_HPI_SPDARAM0_SADDR,HTPX5421Q_HPI_SPDARAM1_EADDR,data_start_p01) !=0

					// now fill-in DARAM/SARAM data memory for DSP-B core
				||  fill_hpi_ram(n,HTPX5421Q_HPI_DARAMB_SADDR,HTPX5421Q_HPI_SARAMB_EADDR,data_start_db) !=0

					// now fill-in the SPDARAM2/3 shared program memory
				||  fill_hpi_ram(n,HTPX5421Q_HPI_SPDARAM2_SADDR,HTPX5421Q_HPI_SPDARAM3_EADDR,data_start_p23) !=0

			 ) 
				goto error;


		   // now increment data and repeat with next DSP core
    	   data_start_da   += data_inc_da;
	       data_start_p01  += data_inc_p01 ;
    	   data_start_db   += data_inc_db ;
	       data_start_p23  += data_inc_p23 ;

    	   data_start_da   &= 0xffff;
	       data_start_p01  &= 0xffff;
    	   data_start_db   &= 0xffff;
	       data_start_p23  &= 0xffff;
		}



	   data_start_da = data_start_da_start;
	   data_start_p01 = data_start_p01_start;
	   data_start_db = data_start_db_start;
	   data_start_p23 = data_start_p23_start;

		// now verify written data
		for(n=0;n<8;n+=2)
		{
			// initilize C54x HPI port 
	   		HTPX5421Q_HPI_INIT(n);

           // verify written DSP on-chip memory
		   if(		verify_hpi_ram(n,HTPX5421Q_HPI_DARAMA_SADDR,HTPX5421Q_HPI_SARAMA_EADDR,data_start_da) !=0
				||  verify_hpi_ram(n,HTPX5421Q_HPI_SPDARAM0_SADDR,HTPX5421Q_HPI_SPDARAM1_EADDR,data_start_p01) !=0
				||  verify_hpi_ram(n,HTPX5421Q_HPI_DARAMB_SADDR,HTPX5421Q_HPI_SARAMB_EADDR,data_start_db) !=0
				||  verify_hpi_ram(n,HTPX5421Q_HPI_SPDARAM2_SADDR,HTPX5421Q_HPI_SPDARAM3_EADDR,data_start_p23) !=0
			 ) 
				goto error;


		   // now increment data and repeat with next DSP core
    	   data_start_da   += data_inc_da;
	       data_start_p01  += data_inc_p01 ;
    	   data_start_db   += data_inc_db ;
	       data_start_p23  += data_inc_p23 ;

    	   data_start_da   &= 0xffff;
	       data_start_p01  &= 0xffff;
    	   data_start_db   &= 0xffff;
	       data_start_p23  &= 0xffff;
		}

	   data_start_da_start = data_start_da ;
	   data_start_p01_start = data_start_p01;
	   data_start_db_start = data_start_db ;
	   data_start_p23_start = data_start_p23;


	   pass_counter++;
 	 }

  error:
    while(1) ;

}


