/***************************************************************
  TORNADO-54x utilities (C54x DSP Environment).
  (C) MicroLAB Systems Ltd, 1997-2001

****************************************************************/

#ifndef __TPX5421Q_DSP_DEF__
#define __TPX5421Q_DSP_DEF__	1


//**** recommended SWWSR/BSCR/SWCR/CLKMD register settings *******

#define TPX5421Q_SWWSR_DATA		0x0000	// 0ws for access to all DSP on-chip RAM
#define TPX5421Q_CLKMD1_DATA	0x07f7	// CPU-A only: required 1st step setting for CLKMD reg (/2)
#define TPX5421Q_CLKMD2_DATA	0x17f7	// CPU-A only: required 2nd step setting for CLKMD reg (PLLx2)
#define TPX5421Q_BSCR_DATA		0x8000	// BSCR reg
#define TPX5421Q_SWCR_DATA		0x0000	// SWCR reg

#define TPX5421Q_PMST_DATA		0xffa8	// default setting for PMST register: MP/MC=0 (!), OVLY=1, DROM=1
//****************************************************************

//------------ general defs --------------------------------------
#define	ON						1
#define	OFF						0
#define	TPX5421Q_ON				1
#define	TPX5421Q_OFF			0

#define	TPX5421Q_XIO_CNF_INPUT	0	// XIO-0/1 pin is configured as OUTPUT
#define	TPX5421Q_XIO_CNF_OUTPUT	1	// XIO-0/1 pin is configured as OUTPUT
#define	TPX5421Q_XIO_CNF_TIMER	2	// XIO-0 only: configured as DSP TIMER output

//----------------------------------------------------------------


//--------------- extended API functions declarations-------------

void	TPX5421Q_init_dsp();

void 	TPX5421Q_set_sx_run(int v);
int 	TPX5421Q_sx_run_data();

void 	TPX5421Q_set_xio0_cnf(int cnf);
void 	TPX5421Q_set_xio1_cnf(int cnf);
int 	TPX5421Q_xio0_cnf_data();
int		TPX5421Q_xio1_cnf_data();
void 	TPX5421Q_set_xio0(int v);
void 	TPX5421Q_set_xio1(int v);
int 	TPX5421Q_xio0_data();
int 	TPX5421Q_xio1_data();

void		TPX5421Q_init_mcbsp2();
unsigned	TPX5421Q_read_mcbsp2_data();
void		TPX5421Q_write_mcbsp2_data(unsigned d);

//----------------------------------------------------------------


/*================ Extended T54x DSP API functions =============================
 Extended TPX5421Q DSP API functions include functions for DSP INIT, SIOX RESET
 control, etc.

 However, in order to reduce the size of DSP code and in order to exclude mutiple 
 function body appearance, any of the Extended API functions can be included/excluded 
 during the compilation process. Inclusion/exclusion of particular Extended API 
 function is controled via global vars, which can be defined in user code to perform 
 a particular action:

	__TPX5421Q_EXCLUDE_LOCAL_FUNCTIONS__ 
		- if defined in user code, then the DSP INIT and I/O control functions
		  are excluded from the user source code. Default is INCLUDED.
*/
#ifndef __TPX5421Q_EXCLUDE_LOCAL_FUNCTIONS__

	//--- local defs (not for user application)
#define	__C54X_SWWSR__	0x28
#define	__C54X_BSCR__	0x29
#define	__C54X_SWCR__	0x2B
#define	__C54X_CLKMD__	0x58
#define	__C54X_HPIC__	0x2C
#define	__C54X_GPIOCR__	0x3C
#define	__C54X_CSIDR__	0x3E

#define	__C542X_GPIOCR_GPIO0_DIR_BITMASK	0x0100	// GPIO-0 direction control bitmask
#define	__C542X_GPIOCR_GPIO0_DIR_OUT		0x0100	// GPIO-0 OUTPUT direction cnf
#define	__C542X_GPIOCR_GPIO0_DATA_BITMASK	0x0001	// GPIO-0 data bitmask
#define	__C542X_GPIOCR_GPIO0_DATA_OFF		0		// GPIO-0 '0' I/O data
#define	__C542X_GPIOCR_GPIO0_DATA_ON		0x0001	// GPIO-0 '1' I/O data

#define	__C542X_GPIOCR_TOUT_EN_BITMASK		0x8000	// bitmask for timer enable at GPIO3 pin
#define	__C542X_GPIOCR_TOUT_EN_OFF			0		// disable timer at GPIO3 pin
#define	__C542X_GPIOCR_TOUT_EN_ON			0x8000	// enable timer at GPIO3 pin

#define	__C542X_GPIOCR_GPIO2_DIR_BITMASK	0x0400	// SXIO-1 (GPIO-2) direction control bitmask
#define	__C542X_GPIOCR_GPIO2_DIR_IN			0		// SXIO-1 (GPIO-2) INPUT direction cnf
#define	__C542X_GPIOCR_GPIO2_DIR_OUT		0x0400	// SXIO-1 (GPIO-2) OUTPUT direction cnf
#define	__C542X_GPIOCR_GPIO3_DIR_BITMASK	0x0800	// SXIO-0 (GPIO-3) direction control bitmask
#define	__C542X_GPIOCR_GPIO3_DIR_IN			0		// SXIO-0 (GPIO-3)  INPUT direction cnf
#define	__C542X_GPIOCR_GPIO3_DIR_OUT		0x0800	// SXIO-0 (GPIO-3)  OUTPUT direction cnf

#define	__C542X_GPIOCR_GPIO2_DATA_BITMASK	0x0004	// SXIO-1 (GPIO-2) data bitmask
#define	__C542X_GPIOCR_GPIO2_DATA_OFF		0		// SXIO-1 (GPIO-2) '0' I/O data
#define	__C542X_GPIOCR_GPIO2_DATA_ON		0x0004	// SXIO-1 (GPIO-2) '1' I/O data
#define	__C542X_GPIOCR_GPIO3_DATA_BITMASK	0x0008	// SXIO-0 (GPIO-3) data bitmask
#define	__C542X_GPIOCR_GPIO3_DATA_OFF		0		// SXIO-0 (GPIO-3) '0' I/O data
#define	__C542X_GPIOCR_GPIO3_DATA_ON		0x0008	// SXIO-0 (GPIO-3) '1' I/O data

#define	__C542X_CSIDR_SUBSYS_ID_BITMASK		0x000F	// CSIDR: subsystem ID mask
#define	__C542X_CSIDR_SUBSYS_A_ID			0x0000	// CSIDR: subsystem-A ID
#define	__C542X_CSIDR_SUBSYS_B_ID			0x0000	// CSIDR: subsystem-B ID

//------------------ DSP init functions --------------------------------
// This function performs initialization of TPX5421Q DSP configiration registers
// and setting SIOX RESET, XIO-0/1 signals.

void	TPX5421Q_init_dsp()
// This fucntion must be used for initialization of PX5421Q DSP core and
// sets SIOX RESET to 'ON'. SXIO-0/1 I/O pins are configured as INPUTS, and
// DSP on-chip timer is disabled.
{ 
	// configure wait state control
  *(unsigned *)__C54X_SWWSR__ 	= TPX5421Q_SWWSR_DATA;
  *(unsigned *)__C54X_SWCR__ 	= TPX5421Q_SWCR_DATA;
  *(unsigned *)__C54X_BSCR__ 	= TPX5421Q_BSCR_DATA;

	// configure GPIO0 pin as output for SIOX RESET control and set it to '1' 
  *(unsigned *)__C54X_GPIOCR__ 	= (__C542X_GPIOCR_GPIO0_DIR_OUT | __C542X_GPIOCR_GPIO0_DATA_ON);

	// configure CLKMD to PLLx2 (100MHz) via CPU-A only
	// note: PLLx2 mode can be set after initial BYPASSx1 mode only via 
	// 		 intermediate setting PLL/2 mode.
  if(((*(unsigned *)__C54X_CSIDR__) & __C542X_CSIDR_SUBSYS_A_ID)== __C542X_CSIDR_SUBSYS_A_ID)
	{
  		*(unsigned *)__C54X_CLKMD__ 	= TPX5421Q_CLKMD1_DATA;
  		*(unsigned *)__C54X_CLKMD__ 	= TPX5421Q_CLKMD2_DATA;
	}

}
//----------------------------------------------------------------------




//------------------- SX-RESET control ---------------------------------
//  1. SIOX RESET is generated as GPIO-0 bit of GPIO-CR register.
//	2. SIOX RESET is set to 'ON' if GPIO-0 is set to the '1' state.


void TPX5421Q_set_sx_run(int v)
// set SIOX RESET line: 
//		v=ON  - set SIOX RESET to 'off', i.e. enables SIOX
//		v=OFF - set SIOX RESET to 'on', i.e. disables SIOX
{
 if(v&1) 
	*(unsigned *)__C54X_GPIOCR__ &= ~__C542X_GPIOCR_GPIO0_DATA_BITMASK;
   else	   
	*(unsigned *)__C54X_GPIOCR__ |= __C542X_GPIOCR_GPIO0_DATA_ON;
}


int TPX5421Q_sx_run_data()
// returns current SIOX RESET line status: 
//		=ON  - SIOX RESET is 'off', i.e. SIOX is enabled
//		=OFF - SIOX RESET is 'on', i.e. SIOX is disabled
{
 return(((*(unsigned *)__C54X_GPIOCR__) & __C542X_GPIOCR_GPIO0_DATA_BITMASK) ? TPX5421Q_OFF:TPX5421Q_ON);
}

//-----------------------------------------------------------------


//----------------- SIOX XIO-0/1 I/O pin control ------------------

void TPX5421Q_set_xio0_cnf(int cnf)
// SIOX XIO-0 pin (DSP GPIO3 pin) can be configured as INPUT, OUTPUT or as
// DSP timer output
{

 if(cnf==TPX5421Q_XIO_CNF_INPUT)
	  {	// configure XIO-0 pin as INPUT and disable TIMER
		*(unsigned *)__C54X_GPIOCR__ 	&= ~__C542X_GPIOCR_GPIO3_DIR_BITMASK;
		*(unsigned *)__C54X_GPIOCR__ 	&= ~__C542X_GPIOCR_TOUT_EN_BITMASK;
	  }
   else
	if(cnf==TPX5421Q_XIO_CNF_OUTPUT)
	  {	// configure XIO-0 pin as OUTPUT  and disable TIMER
		*(unsigned *)__C54X_GPIOCR__ 	|= __C542X_GPIOCR_GPIO3_DIR_OUT;
		*(unsigned *)__C54X_GPIOCR__ 	&= ~__C542X_GPIOCR_TOUT_EN_BITMASK;
  	  }
	 else
	  {	// assume that XIO-0 pin is TIMER output if neither INPUT or OUTPUT are specified
		*(unsigned *)__C54X_GPIOCR__ 	&= ~__C542X_GPIOCR_GPIO3_DIR_BITMASK;
		*(unsigned *)__C54X_GPIOCR__ 	|= __C542X_GPIOCR_TOUT_EN_ON;
  	  }
}


void TPX5421Q_set_xio1_cnf(int cnf)
// SIOX XIO-1 pin (DSP GPIO2 pin) can be configured as INPUT or OUTPUT
{

 if(cnf==TPX5421Q_XIO_CNF_INPUT)
	  {	// configure XIO-1 pin as INPUT and disable TIMER
		*(unsigned *)__C54X_GPIOCR__ 	&= ~__C542X_GPIOCR_GPIO2_DIR_BITMASK;
	  }
   else
	  {	// assume that XIO-1 pin is OUTPUT if non-INPUT is specified
		*(unsigned *)__C54X_GPIOCR__ 	|= __C542X_GPIOCR_GPIO2_DIR_OUT;
  	  }
}



int TPX5421Q_xio0_cnf_data()
// returns XIO-0 (DSP GPIO3) current configuration
{
	// first, check for XIO-0 pin is configured as TIMER output
	if((*(unsigned *)__C54X_GPIOCR__) & __C542X_GPIOCR_TOUT_EN_BITMASK)
			return(TPX5421Q_XIO_CNF_TIMER);

	// if not TIMER outpt, then check for XIO-0 pin is configured as OUTPUT
	// return INPUT CNF otherwise
	if((*(unsigned *)__C54X_GPIOCR__) & __C542X_GPIOCR_GPIO3_DIR_BITMASK)
			return(TPX5421Q_XIO_CNF_OUTPUT);
		else
			return(TPX5421Q_XIO_CNF_INPUT);
}



int TPX5421Q_xio1_cnf_data()
// returns XIO-1 (DSP GPIO2) current configuration
{
	// check for XIO-0 pin is configured as OUTPUT, return INPUT CNF otherwise
	if((*(unsigned *)__C54X_GPIOCR__) & __C542X_GPIOCR_GPIO2_DIR_BITMASK)
			return(TPX5421Q_XIO_CNF_OUTPUT);
		else
			return(TPX5421Q_XIO_CNF_INPUT);
}




void TPX5421Q_set_xio0(int v)
// set XIO-0 (DSP GPIO3) output data for OUTPUT mode
//	v=ON  - set XIO-0 to '1'
//	v=OFF - set XIO-0 to '0'
{
 	if(v&1)  
			*(unsigned *)__C54X_GPIOCR__ |= __C542X_GPIOCR_GPIO3_DATA_ON;
   		else      
			*(unsigned *)__C54X_GPIOCR__ &= ~__C542X_GPIOCR_GPIO3_DATA_BITMASK;
}



int TPX5421Q_xio0_data()
// returns XIO-0 (DSP GPIO3) current data state for INPUT/OUTPUT modes
{
	return(((*(unsigned *)__C54X_GPIOCR__) & __C542X_GPIOCR_GPIO3_DATA_BITMASK)>>3);
}



void TPX5421Q_set_xio1(int v)
// set XIO-1 (DSP GPIO2) output data for OUTPUT mode
//	v=ON  - set XIO-1 to '1'
//	v=OFF - set XIO-2 to '0'
{
 	if(v&1)  
			*(unsigned *)__C54X_GPIOCR__ |= __C542X_GPIOCR_GPIO2_DATA_ON;
   		else      
			*(unsigned *)__C54X_GPIOCR__ &= ~__C542X_GPIOCR_GPIO2_DATA_BITMASK;
}



int TPX5421Q_xio1_data()
// returns XIO-1 (DSP GPIO2) current data state for INPUT/OUTPUT modes
{
	return(((*(unsigned *)__C54X_GPIOCR__) & __C542X_GPIOCR_GPIO2_DATA_BITMASK)>>2);
}

//----------------------------------------------------------------


//------------ McBSP-2 port init and r/w functions ---------------

// recommended McBSP-2 port configuration params 
// notes:
//	1. McBSP-2 port of each DSP core is used locally on-board for 
//     core-to-core communication between different DSP chips:
//		 McBSP2@DSP0A <--> McBSP2@DSP1A
//		 McBSP2@DSP0B <--> McBSP2@DSP2B
//		 McBSP2@DSP2A <--> McBSP2@DSP3A
//		 McBSP2@DSP1B <--> McBSP2@DSP3B
//	2. Configuration below assumes that transmitter of McBSP-2 port
//     is configured to 16-bit data words, maximum CLKX (50MHz), and 
//     generation of FSX on the DX register load. Receiver is configured
//	   to 16-bit data and externally generated CLKR/FSR.

// recommended configuration:
//       XLen=16bits
//       CLKX - Int, 50MHz, active high
//       FSX - output on DX load event, active low (1), advance=+1clkx
//       RLen=16bits
//       CLKR - ext, active high
//       FSR - ext, active low (1), advance=+1clkr

// SPCR: 0000 0000 0000 0000 0000 0000 0000 0000
// PCR:  ---- ---- ---- ---- 0000 1010 0000 1100
// XCR:  0000 0000 0000 0001 0000 0000 0100 0000
// RCR:  0000 0000 0000 0001 0000 0000 0100 0000
// SRGR: 0010 0000 0000 0000 0000 0000 0000 0001

#define	TPX5421Q_MCBSP2_SPCR1_DATA		0x0000
#define	TPX5421Q_MCBSP2_SPCR2_DATA		0x0000
#define	TPX5421Q_MCBSP2_PCR_DATA		0x0a0c
#define	TPX5421Q_MCBSP2_XCR1_DATA		0x0040
#define	TPX5421Q_MCBSP2_XCR2_DATA		0x0001
#define	TPX5421Q_MCBSP2_RCR1_DATA		0x0040
#define	TPX5421Q_MCBSP2_RCR2_DATA		0x0001
#define	TPX5421Q_MCBSP2_SRGR1_DATA		0x0001
#define	TPX5421Q_MCBSP2_SRGR2_DATA		0x2000

#define	TPX5421Q_MCBSP2_SPCR1_GO_MASK	0x0001
#define	TPX5421Q_MCBSP2_SPCR2_GO_MASK	0x0041


// local C54xx McBSP2 port defs (not for customer applications)
#define	__C54X_MCBSP2_DRR2			0x30
#define	__C54X_MCBSP2_DRR1			0x31
#define	__C54X_MCBSP2_DXR2			0x32
#define	__C54X_MCBSP2_DXR1			0x33
#define	__C54X_MCBSP2_SPSA			0x34
#define	__C54X_MCBSP2_SPCR1			0x35
#define	__C54X_MCBSP2_SPCR2			0x35
#define	__C54X_MCBSP2_RCR1			0x35
#define	__C54X_MCBSP2_RCR2			0x35
#define	__C54X_MCBSP2_XCR1			0x35
#define	__C54X_MCBSP2_XCR2			0x35
#define	__C54X_MCBSP2_SRGR1			0x35
#define	__C54X_MCBSP2_SRGR2			0x35
#define	__C54X_MCBSP2_MCR1			0x35
#define	__C54X_MCBSP2_MCR2			0x35
#define	__C54X_MCBSP2_RCERA			0x35
#define	__C54X_MCBSP2_RCERB			0x35
#define	__C54X_MCBSP2_XCERA			0x35
#define	__C54X_MCBSP2_XCERB			0x35
#define	__C54X_MCBSP2_PCR			0x35

#define	__C54X_MCBSP_SBA_SPCR1		0
#define	__C54X_MCBSP_SBA_SPCR2		1
#define	__C54X_MCBSP_SBA_RCR1		2
#define	__C54X_MCBSP_SBA_RCR2		3
#define	__C54X_MCBSP_SBA_XCR1		4
#define	__C54X_MCBSP_SBA_XCR2		5
#define	__C54X_MCBSP_SBA_SRGR1		6
#define	__C54X_MCBSP_SBA_SRGR2		7
#define	__C54X_MCBSP_SBA_MCR1		8
#define	__C54X_MCBSP_SBA_MCR2		9
#define	__C54X_MCBSP_SBA_RCERA		0xa
#define	__C54X_MCBSP_SBA_RCERB		0xb
#define	__C54X_MCBSP_SBA_XCERA		0xc
#define	__C54X_MCBSP_SBA_XCERB		0xd
#define	__C54X_MCBSP_SBA_PCR		0xe


	// configure McBSP-2 port for core-to-core communication
void TPX5421Q_init_mcbsp2()
{
	   *(unsigned *)__C54X_MCBSP2_SPSA  = __C54X_MCBSP_SBA_SPCR1;
	   *(unsigned *)__C54X_MCBSP2_SPCR1 = TPX5421Q_MCBSP2_SPCR1_DATA;
	   *(unsigned *)__C54X_MCBSP2_SPSA  = __C54X_MCBSP_SBA_SPCR2;
	   *(unsigned *)__C54X_MCBSP2_SPCR2 = TPX5421Q_MCBSP2_SPCR2_DATA;

	   *(unsigned *)__C54X_MCBSP2_SPSA  = __C54X_MCBSP_SBA_PCR;
	   *(unsigned *)__C54X_MCBSP2_PCR   = TPX5421Q_MCBSP2_PCR_DATA;

	   *(unsigned *)__C54X_MCBSP2_SPSA  = __C54X_MCBSP_SBA_XCR1;
	   *(unsigned *)__C54X_MCBSP2_XCR1  = TPX5421Q_MCBSP2_XCR1_DATA;
	   *(unsigned *)__C54X_MCBSP2_SPSA  = __C54X_MCBSP_SBA_XCR2;
	   *(unsigned *)__C54X_MCBSP2_XCR2  = TPX5421Q_MCBSP2_XCR2_DATA;

	   *(unsigned *)__C54X_MCBSP2_SPSA  = __C54X_MCBSP_SBA_RCR1;
	   *(unsigned *)__C54X_MCBSP2_XCR1  = TPX5421Q_MCBSP2_RCR1_DATA;
	   *(unsigned *)__C54X_MCBSP2_SPSA  = __C54X_MCBSP_SBA_RCR2;
	   *(unsigned *)__C54X_MCBSP2_XCR2  = TPX5421Q_MCBSP2_RCR2_DATA;

	   *(unsigned *)__C54X_MCBSP2_SPSA  = __C54X_MCBSP_SBA_SRGR1;
	   *(unsigned *)__C54X_MCBSP2_SRGR1 = TPX5421Q_MCBSP2_SRGR1_DATA;
	   *(unsigned *)__C54X_MCBSP2_SPSA  = __C54X_MCBSP_SBA_SRGR2;
	   *(unsigned *)__C54X_MCBSP2_SRGR2 = TPX5421Q_MCBSP2_SRGR2_DATA;


		//-- now enable McBSP port operation
	   *(unsigned *)__C54X_MCBSP2_SPSA  = __C54X_MCBSP_SBA_SPCR1;
	   *(unsigned *)__C54X_MCBSP2_SPCR1 |= TPX5421Q_MCBSP2_SPCR1_GO_MASK;
	   *(unsigned *)__C54X_MCBSP2_SPSA  = __C54X_MCBSP_SBA_SPCR2;
	   *(unsigned *)__C54X_MCBSP2_SPCR2 |= TPX5421Q_MCBSP2_SPCR2_GO_MASK;
		//!!! leave _SPSA pointint to the _SPCR2
		//    for transmitter ready run-time polling
}

#define __TPX5421Q_SET_MCBSP2_SPSA_SPCR2    *(unsigned *)__C54X_MCBSP2_SPSA  = __C54X_MCBSP_SBA_SPCR2
#define __TPX5421Q_SET_MCBSP2_SPSA_SPCR1    *(unsigned *)__C54X_MCBSP2_SPSA  = __C54X_MCBSP_SBA_SPCR1

#define	_TPX5421Q_WAIT_MCBSP2_TRDY			while(!((*(unsigned*)C54X_MCBSP2_SPCR2) & 0x2))
#define	_TPX5421Q_MCBSP2_DX_RG				(*(unsigned*)C54X_MCBSP2_DXR1)

#define	_TPX5421Q_WAIT_MCBSP2_RRDY			while(!((*(unsigned*)C54X_MCBSP2_SPCR1) & 0x2))
#define	_TPX5421Q_MCBSP2_DR_RG				(*(unsigned*)C54X_MCBSP2_DRR1)



void		TPX5421Q_write_mcbsp2_data(unsigned d)
// waits for transmitter ready and writes 16-bit data to McBSP2 transmitter
{
    __TPX5421Q_SET_MCBSP2_SPSA_SPCR2;
	_TPX5421Q_WAIT_MCBSP2_TRDY;
	_TPX5421Q_MCBSP2_DX_RG = d;	
}

unsigned	TPX5421Q_read_mcbsp2_data()
// waits for receiver ready and returns 16-bit data fm McBSP2 receiver
{
    __TPX5421Q_SET_MCBSP2_SPSA_SPCR1;
	_TPX5421Q_WAIT_MCBSP2_RRDY;
	return(_TPX5421Q_MCBSP2_DR_RG);
}


//----------------------------------------------------------------------


//======================================================================
#endif

#endif
