/*******************************************************************************************
  TORNADO-PX5421Q Quad 'C5421 PIOX-16 DSP Coprocessor Utilities. Rev.1A
  Host DSP definition for TORNADO DSP systems/controllers with C3x/C6x DSP.

  (C) MicroLAB Systems Ltd, 2001
  WEB: http://www.mlabsys.com
  FTP: ftp://ftp.mlabsys.com

  Notes:
  ------
    1. This file must be used as .H header file for C3x DSP
       environment of host TORNADO-3x/P3x/E3x/6x/P6x/E6x DSP system/controller.
    2. All commands are adjusted for 32-bit data words and
       occupy least significant 16-bit word.

*******************************************************************************************/

#ifndef __PX5421Q_H3X6X__
#define __PX5421Q_H3X6X__		1

#include	"stdlib.h"


//==================== DSP platform and host PIOX-16 address defs ========================
//	Host TORNADO DSP platform def (_HOST_C6X_DSP_ or _HOST_C3X_DSP_) and
//	PIOX-16 start address (_PIOX16_SADDR_) shall be defined in host TORNADO
//	DSP application prior inclusion of this PX5421QH.H file in order to provide 
//	correct operation of TORNADO-PX5421Q utilities. 
//
//	Either _HOST_C6X_DSP_ or _HOST_C3X_DSP_ constant must be defined to any 
//	value in parent .C file for in order to meet host DSP platform.
//
//	Refer to the .H	file and documentation for your TORNADO-6x/P6x/E6x DSP 
//	system/controller in order to set _PIOX16_SADDR_ to the PIOX-16 base address 
//	value. Host TORNADO-3x/P3x/E3x do not require setting of _PIOX16_SADDR_ def.
//	
//
// note:
//   1. _HOST_C6X_DSP_ constant must be defined to any value in parent .C file for 
//	    any of host T6x/TP6x/E6x DSP platforms. Host TORNADO C6x DSP application
//		must define _PIOX16_SADDR_ in accordance the DSP environment of the
//   	corresponding TORNADO DSP system/controller. In case _PIOX16_SADDR_ is 
//		not defined in host application, then it is	defined in accordance with 
//		the TORNADO-P6x setting.
//   2. _HOST_C3X_DSP_ constant must be defined to any value in parent .C file for 
//	    any of host T3x/TP3x/E3x DSP platforms. Define _PIOX16_SADDR_ to 0xff0000, 
//		since this is common setting for T3x/TP3x/E3x boards.
//   3. In case none of _HOST_C6X_DSP_ and _HOST_C3X_DSP_ is defined
//	    in parent .C file, then it C3x DSP is assumed to be host DSP. 
//		Define _PIOX16_SADDR_ to 0xff0000, since this is common setting for 
//		T3x/TP3x/E3x boards.


	// check for host C6x DSP platform
#ifdef _HOST_C6X_DSP_
		// treat C3X def as erroneous def
#ifdef _HOST_C3X_DSP_		
#undef _HOST_C3X_DSP_		
#endif
		// word address step definition
#define	_WAS_						4		
		// check for _PIOX16_SADDR_ definition. If not defined, set to PIOX-16 base address for TORNADO-P6x.
#ifndef _PIOX16_SADDR_
#define	_PIOX16_SADDR_				0x00200000
#endif
#endif

	// check for host C3x DSP platform
#ifdef _HOST_C3X_DSP_		
		// treat C6X def as erroneous def
#ifdef _HOST_C6X_DSP_		
#undef _HOST_C6X_DSP_		
#endif
		// word address step definition
#define	_WAS_						1		
		// re-define PIOX-16 address to common T3x/TP3x/E3x setting
#ifdef  _PIOX16_SADDR_
#undef	_PIOX16_SADDR_
#endif
#define	_PIOX16_SADDR_				0xff0000	
#endif

	// check for both C3X/C6X defs are undefined, and assume C3X is host DSP
#ifndef _HOST_C6X_DSP_		
#ifndef _HOST_C3X_DSP_		
#define _HOST_C3X_DSP_				1		
		// word address step definition
#define	_WAS_						1		
		// re-define PIOX-16 address to common T3x/TP3x/E3x setting
#ifdef  _PIOX16_SADDR_
#undef	_PIOX16_SADDR_
#endif
#define	_PIOX16_SADDR_				0xff0000	
#endif
#endif
//==========================================================================================


//========================== general defs ==================================================

#define	HTPX5421Q_OFF				0
#define	HTPX5421Q_ON				1

//==========================================================================================


//===================== host PIOX-16 I/O ports definitions =================================

//--- FPGA on-chip registers
#define HTPX5421Q_CNTR1_RG			(_PIOX16_SADDR_+(0x0000*_WAS_))	// r/w: CONTROL #1 register
#define HTPX5421Q_CNTR2_RG			(_PIOX16_SADDR_+(0x0001*_WAS_))	// r/w: CONTROL #2 register

#define HTPX5421Q_HINT_STAT_RG		(_PIOX16_SADDR_+(0x0008*_WAS_))	// r:   HPI HINT status for each C5421 DSP core
#define HTPX5421Q_HPI_ERR_STAT_RG	(_PIOX16_SADDR_+(0x0009*_WAS_))	// r:   HPI timeout status for each C5421 DSP

#define HTPX5421Q_CLR_HPI0_ERR_RG	(_PIOX16_SADDR_+(0x000C*_WAS_))	// w:   clear HPI timeout flag for DSP#0-A/B cores
#define HTPX5421Q_CLR_HPI1_ERR_RG	(_PIOX16_SADDR_+(0x000D*_WAS_))	// w:   clear HPI timeout flag for DSP#1-A/B cores
#define HTPX5421Q_CLR_HPI2_ERR_RG	(_PIOX16_SADDR_+(0x000E*_WAS_))	// w:   clear HPI timeout flag for DSP#2-A/B cores
#define HTPX5421Q_CLR_HPI3_ERR_RG	(_PIOX16_SADDR_+(0x000F*_WAS_))	// w:   clear HPI timeout flag for DSP#3-A/B cores

#define HTPX5421Q_HIRQ_EN_RG		(_PIOX16_SADDR_+(0x0004*_WAS_))	// r/w: Host IRQ Line enable register
#define HTPX5421Q_HIRQ0_IE1_RG		(_PIOX16_SADDR_+(0x0010*_WAS_))	// r/w: Host IRQ-0 Line interrupt mask #1 register
#define HTPX5421Q_HIRQ0_IE2_RG		(_PIOX16_SADDR_+(0x0011*_WAS_))	// r/w: Host IRQ-0 Line interrupt mask #2 register
#define HTPX5421Q_HIRQ1_IE1_RG		(_PIOX16_SADDR_+(0x0012*_WAS_))	// r/w: Host IRQ-1 Line interrupt mask #1 register
#define HTPX5421Q_HIRQ1_IE2_RG		(_PIOX16_SADDR_+(0x0013*_WAS_))	// r/w: Host IRQ-1 Line interrupt mask #2 register
#define HTPX5421Q_HIRQ2_IE1_RG		(_PIOX16_SADDR_+(0x0014*_WAS_))	// r/w: Host IRQ-2 Line interrupt mask #1 register
#define HTPX5421Q_HIRQ2_IE2_RG		(_PIOX16_SADDR_+(0x0015*_WAS_))	// r/w: Host IRQ-2 Line interrupt mask #2 register
#define HTPX5421Q_HIRQ3_IE1_RG		(_PIOX16_SADDR_+(0x0016*_WAS_))	// r/w: Host IRQ-3 Line interrupt mask #1 register
#define HTPX5421Q_HIRQ3_IE2_RG		(_PIOX16_SADDR_+(0x0017*_WAS_))	// r/w: Host IRQ-3 Line interrupt mask #2 register

//--- DSP on-chip HPI ports
#define HTPX5421Q_HPIC0A_RG			(_PIOX16_SADDR_+(0x0020*_WAS_))	// r/w: HPIC reg of C5421#0A DSP core
#define HTPX5421Q_HPID0A_AINC_RG	(_PIOX16_SADDR_+(0x0021*_WAS_))	// r/w: HPID/AINC reg of C5421#0A DSP core
#define HTPX5421Q_HPIA0A_RG			(_PIOX16_SADDR_+(0x0022*_WAS_))	// r/w: HPIA reg of C5421#0A DSP core
#define HTPX5421Q_HPID0A_RG			(_PIOX16_SADDR_+(0x0023*_WAS_))	// r/w: HPID reg of C5421#0A DSP core

#define HTPX5421Q_HPIC0B_RG			(_PIOX16_SADDR_+(0x0024*_WAS_))	// r/w: HPIC reg of C5421#0B DSP core
#define HTPX5421Q_HPID0B_AINC_RG	(_PIOX16_SADDR_+(0x0025*_WAS_))	// r/w: HPID/AINC reg of C5421#0B DSP core
#define HTPX5421Q_HPIA0B_RG			(_PIOX16_SADDR_+(0x0026*_WAS_))	// r/w: HPIA reg of C5421#0B DSP core
#define HTPX5421Q_HPID0B_RG			(_PIOX16_SADDR_+(0x0027*_WAS_))	// r/w: HPID reg of C5421#0B DSP core

#define HTPX5421Q_HPIC1A_RG			(_PIOX16_SADDR_+(0x0028*_WAS_))	// r/w: HPIC reg of C5421#1A DSP core
#define HTPX5421Q_HPID1A_AINC_RG	(_PIOX16_SADDR_+(0x0029*_WAS_))	// r/w: HPID/AINC reg of C5421#1A DSP core
#define HTPX5421Q_HPIA1A_RG			(_PIOX16_SADDR_+(0x002A*_WAS_))	// r/w: HPIA reg of C5421#1A DSP core
#define HTPX5421Q_HPID1A_RG			(_PIOX16_SADDR_+(0x002B*_WAS_))	// r/w: HPID reg of C5421#1A DSP core

#define HTPX5421Q_HPIC1B_RG			(_PIOX16_SADDR_+(0x002C*_WAS_))	// r/w: HPIC reg of C5421#1B DSP core
#define HTPX5421Q_HPID1B_AINC_RG	(_PIOX16_SADDR_+(0x002D*_WAS_))	// r/w: HPID/AINC reg of C5421#1B DSP core
#define HTPX5421Q_HPIA1B_RG			(_PIOX16_SADDR_+(0x002E*_WAS_))	// r/w: HPIA reg of C5421#1B DSP core
#define HTPX5421Q_HPID1B_RG			(_PIOX16_SADDR_+(0x002F*_WAS_))	// r/w: HPID reg of C5421#1B DSP core

#define HTPX5421Q_HPIC2A_RG			(_PIOX16_SADDR_+(0x0030*_WAS_))	// r/w: HPIC reg of C5421#2A DSP core
#define HTPX5421Q_HPID2A_AINC_RG	(_PIOX16_SADDR_+(0x0031*_WAS_))	// r/w: HPID/AINC reg of C5421#2A DSP core
#define HTPX5421Q_HPIA2A_RG			(_PIOX16_SADDR_+(0x0032*_WAS_))	// r/w: HPIA reg of C5421#2A DSP core
#define HTPX5421Q_HPID2A_RG			(_PIOX16_SADDR_+(0x0033*_WAS_))	// r/w: HPID reg of C5421#2A DSP core

#define HTPX5421Q_HPIC2B_RG			(_PIOX16_SADDR_+(0x0034*_WAS_))	// r/w: HPIC reg of C5421#2B DSP core
#define HTPX5421Q_HPID2B_AINC_RG	(_PIOX16_SADDR_+(0x0035*_WAS_))	// r/w: HPID/AINC reg of C5421#2B DSP core
#define HTPX5421Q_HPIA2B_RG			(_PIOX16_SADDR_+(0x0036*_WAS_))	// r/w: HPIA reg of C5421#2B DSP core
#define HTPX5421Q_HPID2B_RG			(_PIOX16_SADDR_+(0x0037*_WAS_))	// r/w: HPID reg of C5421#2B DSP core

#define HTPX5421Q_HPIC3A_RG			(_PIOX16_SADDR_+(0x0038*_WAS_))	// r/w: HPIC reg of C5421#3A DSP core
#define HTPX5421Q_HPID3A_AINC_RG	(_PIOX16_SADDR_+(0x0039*_WAS_))	// r/w: HPID/AINC reg of C5421#3A DSP core
#define HTPX5421Q_HPIA3A_RG			(_PIOX16_SADDR_+(0x003A*_WAS_))	// r/w: HPIA reg of C5421#3A DSP core
#define HTPX5421Q_HPID3A_RG			(_PIOX16_SADDR_+(0x003B*_WAS_))	// r/w: HPID reg of C5421#3A DSP core

#define HTPX5421Q_HPIC3B_RG			(_PIOX16_SADDR_+(0x003C*_WAS_))	// r/w: HPIC reg of C5421#3B DSP core
#define HTPX5421Q_HPID3B_AINC_RG	(_PIOX16_SADDR_+(0x003D*_WAS_))	// r/w: HPID/AINC reg of C5421#3B DSP core
#define HTPX5421Q_HPIA3B_RG			(_PIOX16_SADDR_+(0x003E*_WAS_))	// r/w: HPIA reg of C5421#3B DSP core
#define HTPX5421Q_HPID3B_RG			(_PIOX16_SADDR_+(0x003F*_WAS_))	// r/w: HPID reg of C5421#3B DSP core

//==========================================================================================


//======================== host PIOX-16 register data definitions ==========================

#define HTPX5421Q_RG_DATA_BITMASK			0xff	// D0..D7 only are valid for FPGA on-chip registers	

	//-- Control #1 Register bits
#define HTPX5421Q_DSP0A_MRESET_RESET 		0x0		// DSP#0A RESET control
#define HTPX5421Q_DSP0A_MRESET_GO			0x1
#define HTPX5421Q_DSP0A_MRESET_BITMASK		0x1
#define HTPX5421Q_DSP0B_MRESET_RESET 		0x0		// DSP#0B RESET control
#define HTPX5421Q_DSP0B_MRESET_GO			0x2
#define HTPX5421Q_DSP0B_MRESET_BITMASK		0x2

#define HTPX5421Q_DSP1A_MRESET_RESET		0x0		// DSP#1A RESET control
#define HTPX5421Q_DSP1A_MRESET_GO			0x4
#define HTPX5421Q_DSP1A_MRESET_BITMASK		0x4
#define HTPX5421Q_DSP1B_MRESET_RESET 		0x0		// DSP#1B RESET control
#define HTPX5421Q_DSP1B_MRESET_GO			0x8
#define HTPX5421Q_DSP1B_MRESET_BITMASK		0x8

#define HTPX5421Q_DSP2A_MRESET_RESET 		0x00	// DSP#2A RESET control
#define HTPX5421Q_DSP2A_MRESET_GO			0x10
#define HTPX5421Q_DSP2A_MRESET_BITMASK		0x10
#define HTPX5421Q_DSP2B_MRESET_RESET		0x00	// DSP#2B RESET control
#define HTPX5421Q_DSP2B_MRESET_GO			0x20
#define HTPX5421Q_DSP2B_MRESET_BITMASK		0x20

#define HTPX5421Q_DSP3A_MRESET_RESET 		0x00	// DSP#3A RESET control
#define HTPX5421Q_DSP3A_MRESET_GO			0x40
#define HTPX5421Q_DSP3A_MRESET_BITMASK		0x40
#define HTPX5421Q_DSP3B_MRESET_RESET 		0x00	// DSP#3B RESET control
#define HTPX5421Q_DSP3B_MRESET_GO			0x80
#define HTPX5421Q_DSP3B_MRESET_BITMASK		0x80

#define HTPX5421Q_DSP_MRESET_RESET_ALL 		0x00	// value to put all DSP to the RESET state
#define HTPX5421Q_DSP_MRESET_GO_ALL 		0xff	// value to put all DSP to the RUN state

		// uniform aliases for DSP RESET bits of CONTROL#1 register
#define HTPX5421Q_DSP0A_RESET_ON			0x0		// DSP#0A RESET control
#define HTPX5421Q_DSP0A_RESET_OFF			0x1
#define HTPX5421Q_DSP0A_RESET_BITMASK		0x1
#define HTPX5421Q_DSP0B_RESET_ON	 		0x0		// DSP#0B RESET control
#define HTPX5421Q_DSP0B_RESET_OFF			0x2
#define HTPX5421Q_DSP0B_RESET_BITMASK		0x2

#define HTPX5421Q_DSP1A_RESET_ON			0x0		// DSP#1A RESET control
#define HTPX5421Q_DSP1A_RESET_OFF			0x4
#define HTPX5421Q_DSP1A_RESET_BITMASK		0x4
#define HTPX5421Q_DSP1B_RESET_ON	 		0x0		// DSP#1B RESET control
#define HTPX5421Q_DSP1B_RESET_OFF			0x8
#define HTPX5421Q_DSP1B_RESET_BITMASK		0x8

#define HTPX5421Q_DSP2A_RESET_ON			0x00	// DSP#2A RESET control
#define HTPX5421Q_DSP2A_RESET_OFF			0x10
#define HTPX5421Q_DSP2A_RESET_BITMASK		0x10
#define HTPX5421Q_DSP2B_RESET_ON			0x00	// DSP#2B RESET control
#define HTPX5421Q_DSP2B_RESET_OFF			0x20
#define HTPX5421Q_DSP2B_RESET_BITMASK		0x20

#define HTPX5421Q_DSP3A_RESET_ON 			0x00	// DSP#3A RESET control
#define HTPX5421Q_DSP3A_RESET_OFF			0x40
#define HTPX5421Q_DSP3A_RESET_BITMASK		0x40
#define HTPX5421Q_DSP3B_RESET_ON	 		0x00	// DSP#3B RESET control
#define HTPX5421Q_DSP3B_RESET_OFF			0x80
#define HTPX5421Q_DSP3B_RESET_BITMASK		0x80

#define HTPX5421Q_DSP_RESET_ON_ALL	 		0x00	// value to put all DSP to the RESET state
#define HTPX5421Q_DSP_RESET_OFF_ALL 		0xff	// value to put all DSP to the RUN state


	//-- Control #2 Register bits
#define HTPX5421Q_HPI_RESET_RESET			0x0		// common HPI RESET for all DSP cores
#define HTPX5421Q_HPI_RESET_GO				0x1
#define HTPX5421Q_HPI_RESET_BITMASK			0x1
		// uniform aliases
#define HTPX5421Q_HPI_RESET_ON				0x0		
#define HTPX5421Q_HPI_RESET_OFF				0x1

#define HTPX5421Q_HPI_ERR_EN_ON				0x80	// enable HPI TIMEOUT error control
#define HTPX5421Q_HPI_ERR_EN_OFF			0x0
#define HTPX5421Q_HPI_ERR_EN_BITMASK		0x80


	//-- DSP HINT status Register bits
#define HTPX5421Q_HINT0A_BITMASK			0x1		// HINT bitmask/defs for DSP#0A core
#define HTPX5421Q_HINT0A_OFF				0x0
#define HTPX5421Q_HINT0A_ON					0x1
#define HTPX5421Q_HINT0B_BITMASK			0x2		// HINT bitmask/defs for DSP#0B core
#define HTPX5421Q_HINT0B_OFF				0x0
#define HTPX5421Q_HINT0B_ON					0x2

#define HTPX5421Q_HINT1A_BITMASK			0x4		// HINT bitmask/defs for DSP#1A core
#define HTPX5421Q_HINT1A_OFF				0x0
#define HTPX5421Q_HINT1A_ON					0x4
#define HTPX5421Q_HINT1B_BITMASK			0x8		// HINT bitmask/defs for DSP#1B core
#define HTPX5421Q_HINT1B_OFF				0x0
#define HTPX5421Q_HINT1B_ON					0x8

#define HTPX5421Q_HINT2A_BITMASK			0x10	// HINT bitmask/defs for DSP#2A core
#define HTPX5421Q_HINT2A_OFF				0x0
#define HTPX5421Q_HINT2A_ON					0x10
#define HTPX5421Q_HINT2B_BITMASK			0x20	// HINT bitmask/defs for DSP#2B core
#define HTPX5421Q_HINT2B_OFF				0x0
#define HTPX5421Q_HINT2B_ON					0x20

#define HTPX5421Q_HINT3A_BITMASK			0x40	// HINT bitmask/defs for DSP#3A core
#define HTPX5421Q_HINT3A_OFF				0x0
#define HTPX5421Q_HINT3A_ON					0x40
#define HTPX5421Q_HINT3B_BITMASK			0x80	// HINT bitmask/defs for DSP#3B core
#define HTPX5421Q_HINT3B_OFF				0x0
#define HTPX5421Q_HINT3B_ON					0x80


	//-- DSP HPI ERROR status Register bits
#define HTPX5421Q_HPI0_ERR_BITMASK			0x1		// HPI access timeout bitmask/defs for DSP#0-A/B cores
#define HTPX5421Q_HPI0_ERR_OFF				0x0
#define HTPX5421Q_HPI0_ERR_ON				0x1

#define HTPX5421Q_HPI1_ERR_BITMASK			0x2		// HPI access timeout bitmask/defs for DSP#1-A/B cores
#define HTPX5421Q_HPI1_ERR_OFF				0x0
#define HTPX5421Q_HPI1_ERR_ON				0x2

#define HTPX5421Q_HPI2_ERR_BITMASK			0x4		// HPI access timeout bitmask/defs for DSP#2-A/B cores
#define HTPX5421Q_HPI2_ERR_OFF				0x0
#define HTPX5421Q_HPI2_ERR_ON				0x4

#define HTPX5421Q_HPI3_ERR_BITMASK			0x8		// HPI access timeout bitmask/defs for DSP#3-A/B cores
#define HTPX5421Q_HPI3_ERR_OFF				0x0
#define HTPX5421Q_HPI3_ERR_ON				0x8


	//-- PIOX HIRQ-0..3 line enable Register bits
#define HTPX5421Q_HIRQ0_EN_ON				0x1		// HIRQ0 bitmask/defs
#define HTPX5421Q_HIRQ0_EN_OFF				0x0
#define HTPX5421Q_HIRQ0_EN_BITMASK			0x1

#define HTPX5421Q_HIRQ1_EN_ON				0x2		// HIRQ1bitmask/defs
#define HTPX5421Q_HIRQ1_EN_OFF				0x0
#define HTPX5421Q_HIRQ1_EN_BITMASK			0x2

#define HTPX5421Q_HIRQ2_EN_ON				0x4		// HIRQ2 bitmask/defs
#define HTPX5421Q_HIRQ2_EN_OFF				0x0
#define HTPX5421Q_HIRQ2_EN_BITMASK			0x4

#define HTPX5421Q_HIRQ3_EN_ON				0x8		// HIRQ3 bitmask/defs
#define HTPX5421Q_HIRQ3_EN_OFF				0x0
#define HTPX5421Q_HIRQ3_EN_BITMASK			0x8

#define HTPX5421Q_HIRQ_EN_ON_ALL			0xf		// value to enable all HIRQs
#define HTPX5421Q_HIRQ_EN_OFF_ALL			0x0		// value to disable all HIRQs

	//-- host PIOX-16 HIRQ-0..3 IE Register bits
#define HTPX5421Q_HINT0A_IE_ON				0x1		// IE1 rg: HINT IE bitmask/def for DSP#0-A/B cores
#define HTPX5421Q_HINT0A_IE_OFF				0x0
#define HTPX5421Q_HINT0A_IE_BITMASK			0x1
#define HTPX5421Q_HINT0B_IE_ON				0x2
#define HTPX5421Q_HINT0B_IE_OFF				0x0
#define HTPX5421Q_HINT0B_IE_BITMASK			0x2

#define HTPX5421Q_HINT1A_IE_ON				0x4		// IE1 rg: HINT IE bitmask/def for DSP#1-A/B cores
#define HTPX5421Q_HINT1A_IE_OFF				0x0
#define HTPX5421Q_HINT1A_IE_BITMASK			0x4
#define HTPX5421Q_HINT1B_IE_ON				0x8
#define HTPX5421Q_HINT1B_IE_OFF				0x0
#define HTPX5421Q_HINT1B_IE_BITMASK			0x8

#define HTPX5421Q_HINT2A_IE_ON				0x10	// IE1 rg: HINT IE bitmask/def for DSP#2-A/B cores
#define HTPX5421Q_HINT2A_IE_OFF				0x0
#define HTPX5421Q_HINT2A_IE_BITMASK			0x10
#define HTPX5421Q_HINT2B_IE_ON				0x20
#define HTPX5421Q_HINT2B_IE_OFF				0x0
#define HTPX5421Q_HINT2B_IE_BITMASK			0x20

#define HTPX5421Q_HINT3A_IE_ON				0x40	// IE1 rg: HINT IE bitmask/def for DSP#3-A/B cores
#define HTPX5421Q_HINT3A_IE_OFF				0x0
#define HTPX5421Q_HINT3A_IE_BITMASK			0x40
#define HTPX5421Q_HINT3B_IE_ON				0x80
#define HTPX5421Q_HINT3B_IE_OFF				0x0
#define HTPX5421Q_HINT3B_IE_BITMASK			0x80

#define HTPX5421Q_HINT_IE_ON_ALL			0xff	// IE1 rg: value to enable IRQs fm all DSP cores
#define HTPX5421Q_HINT_IE_OFF_ALL			0x0		// IE1 rg: value to disable IRQs fm all DSP cores

#define HTPX5421Q_HPI_ERR_IE_ON				0x80	// IE2 rg: summary HPI error IE bitmask/defs
#define HTPX5421Q_HPI_ERR_IE_OFF			0x0
#define HTPX5421Q_HPI_ERR_IE_BITMASK		0x80



	//-- 'C5421 DSP HPI16 defs
#define HTPX5421Q_HPI_DATA_BITMASK  		0xffff	//16-bit bitmask for HPIC/HPIA/HPID regs of C5421 HPI16 port

		// HPIC register bitmasks/defs
#define	HTPX5421Q_HPIC_HINT_BITMASK			0x4		// HINT: r-only, write '1' in order to reset HINT
#define	HTPX5421Q_HPIC_HINT_OFF				0
#define	HTPX5421Q_HPIC_HINT_ON				0x4
#define	HTPX5421Q_HPIC_XHPIA_BITMASK		0x20	// XHPIA: r/w
#define	HTPX5421Q_HPIC_XHPIA_OFF			0
#define	HTPX5421Q_HPIC_XHPIA_ON				0x20
#define	HTPX5421Q_HPIC_HRDY_BITMASK			0x8		// HRDY: r-only
#define	HTPX5421Q_HPIC_HRDY_OFF				0
#define	HTPX5421Q_HPIC_HRDY_ON				0x8		
#define HTPX5421Q_HPIC_RD_BITMASK	  		0x2c

#define	HTPX5421Q_HPIC_WR_HINT_OFF			0x4		// HINT:   write '1' in order to clear HINT
#define	HTPX5421Q_HPIC_WR_DSPINT_ON			0x2		// DSPINT: w-only (write '1' in order to set DSPINT)
#define	HTPX5421Q_HPIC_WR_FETCH_ON			0x10	// FETCH:  w-only (write '1' in order to set FETCH)
#define HTPX5421Q_HPIC_WR_BITMASK	  		0x36


		// valid HPI address ranges for C5421 DSP
		// Note !!!: CPU-A and CPU-B cores can be accessed via both HPI-A and HPI-B areas, 
		// 			 however 'cross-routed' accesses are performed with extra 6 access cycles.
			// general HPI areas def
#define	HTPX5421Q_HPI_AREA1_SADDR			0x00060	// area#1 (CPU-A data, program shared RAM #0/#1)
#define	HTPX5421Q_HPI_AREA1_EADDR			0x0ffff
#define	HTPX5421Q_HPI_AREA1_LEN				0x1ffA0
#define	HTPX5421Q_HPI_AREA2_SADDR			0x20060	// area#2 (CPU-B data, program shared RAM #2/#3)	
#define	HTPX5421Q_HPI_AREA2_EADDR			0x3ffff
#define	HTPX5421Q_HPI_AREA2_LEN				0x1ffA0
#define	HTPX5421Q_HPI_AREAS					2		// number of HPI areas
#define	HTPX5421Q_HPI_PAGE_LEN				0x10000	// HPI page length. HPI page must be selected via XHPIA bit.

			// DSP core (#A/#B) specific HPIA mapping (MMREgs are not included)
#define	HTPX5421Q_HPI_DARAMA_SADDR			0x00060	// CPU-A: data/program dual access RAM (DARAM) at 0x0060..0x7fff@D/P
#define	HTPX5421Q_HPI_DARAMA_EADDR			0x07fff	
#define	HTPX5421Q_HPI_DARAMA_LEN			0x07fa0	
#define	HTPX5421Q_HPI_SARAMA_SADDR			0x08000	// CPU-A: optional data RAM (SARAM) at 0x8000..0xffff@D
#define	HTPX5421Q_HPI_SARAMA_EADDR			0x0ffff	
#define	HTPX5421Q_HPI_SARAMA_LEN			0x08000	

#define	HTPX5421Q_HPI_SPDARAM0_SADDR		0x10000	// CPU-A/B: shared program dual-access RAM (SDARAM0) at 0x08000..0x0ffff@P
#define	HTPX5421Q_HPI_SPDARAM0_EADDR		0x17fff	
#define	HTPX5421Q_HPI_SPDARAM0_LEN			0x08000	
#define	HTPX5421Q_HPI_SPDARAM1_SADDR		0x18000	// CPU-A/B: shared program dual-access RAM (SDARAM1) at 0x18000..0x1ffff@P
#define	HTPX5421Q_HPI_SPDARAM1_EADDR		0x1ffff	
#define	HTPX5421Q_HPI_SPDARAM1_LEN			0x08000	

#define	HTPX5421Q_HPI_DARAMB_SADDR			0x20060	// CPU-B: data/program dual access RAM (DARAM) at 0x0060..0x7fff@D/P
#define	HTPX5421Q_HPI_DARAMB_EADDR			0x27fff	
#define	HTPX5421Q_HPI_DARAMB_LEN			0x07fa0	
#define	HTPX5421Q_HPI_SARAMB_SADDR			0x28000	// CPU-B: optional data RAM (SARAM) at 0x8000..0xffff@D
#define	HTPX5421Q_HPI_SARAMB_EADDR			0x2ffff	
#define	HTPX5421Q_HPI_SARAMB_LEN			0x08000	

#define	HTPX5421Q_HPI_SPDARAM2_SADDR		0x30000	// CPU-A/B: shared program dual-access RAM (SDARAM2) at 0x28000..0x2ffff@P
#define	HTPX5421Q_HPI_SPDARAM2_EADDR		0x37fff	
#define	HTPX5421Q_HPI_SPDARAM2_LEN			0x08000	
#define	HTPX5421Q_HPI_SPDARAM3_SADDR		0x38000	// CPU-A/B: shared program dual-access RAM (SDARAM3) at 0x38000..0x3ffff@P
#define	HTPX5421Q_HPI_SPDARAM3_EADDR		0x3ffff	
#define	HTPX5421Q_HPI_SPDARAM3_LEN			0x08000	

#define	HTPX5421Q_HPI_DARAMA_BADDR			0x00000	// CPU-A: base addr for data/program dual access RAM (DARAM)
#define	HTPX5421Q_HPI_DARAMB_BADDR			0x20000	// CPU-B: base addr for data/program dual access RAM (DARAM)
#define	HTPX5421Q_HPI_SPDARAM0_SADDR		0x10000	// CPU-A/B: base addr for shared program dual-access RAM (SDARAM0)
#define	HTPX5421Q_HPI_SPDARAM1_SADDR		0x18000	// CPU-A/B: base addr for shared program dual-access RAM (SDARAM1)
#define	HTPX5421Q_HPI_SPDARAM2_SADDR		0x30000	// CPU-A/B: base addr for shared program dual-access RAM (SDARAM2)
#define	HTPX5421Q_HPI_SPDARAM3_SADDR		0x38000	// CPU-A/B: base addr for shared program dual-access RAM (SDARAM3)


			// DSP core on-chip address mapping
#define	HTPX5421Q_DSP_MMREG_SADDR			0x00000	// mmregs (CPU+peripheral)
#define	HTPX5421Q_DSP_MMREG_EADDR			0x0005f	
#define	HTPX5421Q_DSP_MMREG_LEN				0x00060	
#define	HTPX5421Q_DSP_DARAM_SADDR			0x00060	// data/program dual access RAM (DARAM) at 0x0060..0x7fff@D/P
#define	HTPX5421Q_DSP_DARAM_EADDR			0x07fff	
#define	HTPX5421Q_DSP_DARAM_LEN				0x07fa0	
#define	HTPX5421Q_DSP_SARAM_SADDR			0x08000	// optional data RAM (SARAM) at 0x8000..0xffff@D
#define	HTPX5421Q_DSP_SARAM_EADDR			0x0ffff	
#define	HTPX5421Q_DSP_SARAM_LEN				0x08000	
#define	HTPX5421Q_DSP_DRAM_PAGE_LEN			0x10000	

#define	HTPX5421Q_DSP_SPDARAM0_SADDR		0x08000	// shared program dual-access RAM (SDARAM0) at 0x08000..0x0ffff@P
#define	HTPX5421Q_DSP_SPDARAM0_EADDR		0x0ffff	
#define	HTPX5421Q_DSP_SPDARAM0_LEN			0x08000	
#define	HTPX5421Q_DSP_SPDARAM1_SADDR		0x18000	// shared program dual-access RAM (SDARAM1) at 0x18000..0x1ffff@P
#define	HTPX5421Q_DSP_SPDARAM1_EADDR		0x1ffff	
#define	HTPX5421Q_DSP_SPDARAM1_LEN			0x08000	

#define	HTPX5421Q_DSP_SPDARAM2_SADDR		0x28000	// shared program dual-access RAM (SDARAM2) at 0x28000..0x2ffff@P
#define	HTPX5421Q_DSP_SPDARAM2_EADDR		0x2ffff	
#define	HTPX5421Q_DSP_SPDARAM2_LEN			0x08000	
#define	HTPX5421Q_DSP_SPDARAM3_SADDR		0x38000	// shared program dual-access RAM (SDARAM3) at 0x38000..0x3ffff@P
#define	HTPX5421Q_DSP_SPDARAM3_EADDR		0x3ffff	
#define	HTPX5421Q_DSP_SPDARAM3_LEN			0x08000	
#define	HTPX5421Q_DSP_PRAM_PAGE_LEN			0x10000	


//==========================================================================================



//===================== macros for accessing HIF registers =================================
	// these IDs for DSP cores shall be used as 'N' parameter for DSP MRESET control macros
	// and for universal HPI access macros
#define	HTPX5421Q_DSP0A						0
#define	HTPX5421Q_DSP0B						1
#define	HTPX5421Q_DSP1A						2
#define	HTPX5421Q_DSP1B						3
#define	HTPX5421Q_DSP2A						4
#define	HTPX5421Q_DSP2B						5
#define	HTPX5421Q_DSP3A						6
#define	HTPX5421Q_DSP3B						7

	// these IDs for DSP HINTs shall be used as 'N' parameter for universal HINT_DATA access macros
#define	HTPX5421Q_HINT0A					0
#define	HTPX5421Q_HINT0B					1
#define	HTPX5421Q_HINT1A					2
#define	HTPX5421Q_HINT1B					3
#define	HTPX5421Q_HINT2A					4
#define	HTPX5421Q_HINT2B					5
#define	HTPX5421Q_HINT3A					6
#define	HTPX5421Q_HINT3B					7

	// these IDs for HPI#n error shall be used as 'N' parameter for universal HPI_ERR_DATA and CLR_HPI_ERR macros
#define	HTPX5421Q_HPI0_ERR					0
#define	HTPX5421Q_HPI1_ERR					1
#define	HTPX5421Q_HPI2_ERR					2
#define	HTPX5421Q_HPI3_ERR					3


// these IDs for HIRQ shall be used as 'N' parameter for all HIRQ_IE access macros
#define	HTPX5421Q_HIRQ0						0
#define	HTPX5421Q_HIRQ1						1
#define	HTPX5421Q_HIRQ2						2
#define	HTPX5421Q_HIRQ3						3

// these IDs for DSP HINTs and HPI_ERROR shall be used as 'M' parameter for universal HIRQ_IE access macros
#define	HTPX5421Q_HINT0A_IE					0
#define	HTPX5421Q_HINT0B_IE					1
#define	HTPX5421Q_HINT1A_IE					2
#define	HTPX5421Q_HINT1B_IE					3
#define	HTPX5421Q_HINT2A_IE					4
#define	HTPX5421Q_HINT2B_IE					5
#define	HTPX5421Q_HINT3A_IE					6
#define	HTPX5421Q_HINT3B_IE					7
#define	HTPX5421Q_HPI_ERR_IE				8



	//--- CONTROL-1 register
#define HTPX5421Q_CNTR1_RG_DATA				((*(unsigned *)HTPX5421Q_CNTR1_RG) & HTPX5421Q_RG_DATA_BITMASK)
#define HTPX5421Q_SET_CNTR1_RG(d)			(*(unsigned *)HTPX5421Q_CNTR1_RG) = d

#define HTPX5421Q_DSP0A_MRESET_DATA			(HTPX5421Q_CNTR1_RG_DATA & HTPX5421Q_DSP0A_MRESET_BITMASK)
#define HTPX5421Q_SET_DSP0A_MRESET			HTPX5421Q_SET_CNTR1_RG(HTPX5421Q_CNTR1_RG_DATA & ~HTPX5421Q_DSP0A_MRESET_BITMASK)
#define HTPX5421Q_SET_DSP0A_MGO				HTPX5421Q_SET_CNTR1_RG(HTPX5421Q_CNTR1_RG_DATA | HTPX5421Q_DSP0A_MRESET_GO)

#define HTPX5421Q_DSP0B_MRESET_DATA			(HTPX5421Q_CNTR1_RG_DATA & HTPX5421Q_DSP0B_MRESET_BITMASK)
#define HTPX5421Q_SET_DSP0B_MRESET			HTPX5421Q_SET_CNTR1_RG(HTPX5421Q_CNTR1_RG_DATA & ~HTPX5421Q_DSP0B_MRESET_BITMASK) 
#define HTPX5421Q_SET_DSP0B_MGO				HTPX5421Q_SET_CNTR1_RG(HTPX5421Q_CNTR1_RG_DATA | HTPX5421Q_DSP0B_MRESET_GO)

#define HTPX5421Q_DSP1A_MRESET_DATA			(HTPX5421Q_CNTR1_RG_DATA & HTPX5421Q_DSP1A_MRESET_BITMASK)
#define HTPX5421Q_SET_DSP1A_MRESET			HTPX5421Q_SET_CNTR1_RG(HTPX5421Q_CNTR1_RG_DATA & ~HTPX5421Q_DSP1A_MRESET_BITMASK) 
#define HTPX5421Q_SET_DSP1A_MGO				HTPX5421Q_SET_CNTR1_RG(HTPX5421Q_CNTR1_RG_DATA | HTPX5421Q_DSP1A_MRESET_GO)

#define HTPX5421Q_DSP1B_MRESET_DATA			(HTPX5421Q_CNTR1_RG_DATA & HTPX5421Q_DSP1B_MRESET_BITMASK)
#define HTPX5421Q_SET_DSP1B_MRESET			HTPX5421Q_SET_CNTR1_RG(HTPX5421Q_CNTR1_RG_DATA & ~HTPX5421Q_DSP1B_MRESET_BITMASK) 
#define HTPX5421Q_SET_DSP1B_MGO				HTPX5421Q_SET_CNTR1_RG(HTPX5421Q_CNTR1_RG_DATA | HTPX5421Q_DSP1B_MRESET_GO)

#define HTPX5421Q_DSP2A_MRESET_DATA			(HTPX5421Q_CNTR1_RG_DATA & HTPX5421Q_DSP2A_MRESET_BITMASK)
#define HTPX5421Q_SET_DSP2A_MRESET			HTPX5421Q_SET_CNTR1_RG(HTPX5421Q_CNTR1_RG_DATA & ~HTPX5421Q_DSP2A_MRESET_BITMASK) 
#define HTPX5421Q_SET_DSP2A_MGO				HTPX5421Q_SET_CNTR1_RG(HTPX5421Q_CNTR1_RG_DATA | HTPX5421Q_DSP2A_MRESET_GO)

#define HTPX5421Q_DSP2B_MRESET_DATA			(HTPX5421Q_CNTR1_RG_DATA & HTPX5421Q_DSP2B_MRESET_BITMASK)
#define HTPX5421Q_SET_DSP2B_MRESET			HTPX5421Q_SET_CNTR1_RG(HTPX5421Q_CNTR1_RG_DATA & ~HTPX5421Q_DSP2B_MRESET_BITMASK) 
#define HTPX5421Q_SET_DSP2B_MGO				HTPX5421Q_SET_CNTR1_RG(HTPX5421Q_CNTR1_RG_DATA | HTPX5421Q_DSP2B_MRESET_GO)

#define HTPX5421Q_DSP3A_MRESET_DATA			(HTPX5421Q_CNTR1_RG_DATA & HTPX5421Q_DSP3A_MRESET_BITMASK)
#define HTPX5421Q_SET_DSP3A_MRESET			HTPX5421Q_SET_CNTR1_RG(HTPX5421Q_CNTR1_RG_DATA & ~HTPX5421Q_DSP3A_MRESET_BITMASK) 
#define HTPX5421Q_SET_DSP3A_MGO				HTPX5421Q_SET_CNTR1_RG(HTPX5421Q_CNTR1_RG_DATA | HTPX5421Q_DSP3A_MRESET_GO)

#define HTPX5421Q_DSP3B_MRESET_DATA			(HTPX5421Q_CNTR1_RG_DATA & HTPX5421Q_DSP3B_MRESET_BITMASK)
#define HTPX5421Q_SET_DSP3B_MRESET			HTPX5421Q_SET_CNTR1_RG(HTPX5421Q_CNTR1_RG_DATA & ~HTPX5421Q_DSP3B_MRESET_BITMASK) 
#define HTPX5421Q_SET_DSP3B_MGO				HTPX5421Q_SET_CNTR1_RG(HTPX5421Q_CNTR1_RG_DATA | HTPX5421Q_DSP3B_MRESET_GO)

#define HTPX5421Q_SET_DSP_MRESET_ALL		HTPX5421Q_SET_CNTR1_RG(HTPX5421Q_DSP_MRESET_RESET_ALL) 
#define HTPX5421Q_SET_DSP_MGO_ALL			HTPX5421Q_SET_CNTR1_RG(HTPX5421Q_DSP_MRESET_GO_ALL)

			// uniform aliases for DSP RESET control macros
#define HTPX5421Q_DSP0A_RESET_DATA			HTPX5421Q_DSP0A_MRESET_DATA
#define HTPX5421Q_SET_DSP0A_RESET_ON		HTPX5421Q_SET_DSP0A_MRESET			
#define HTPX5421Q_SET_DSP0A_RESET_OFF		HTPX5421Q_SET_DSP0A_MGO			

#define HTPX5421Q_DSP0B_RESET_DATA			HTPX5421Q_DSP0B_MRESET_DATA			
#define HTPX5421Q_SET_DSP0B_RESET_ON		HTPX5421Q_SET_DSP0B_MRESET
#define HTPX5421Q_SET_DSP0B_RESET_OFF		HTPX5421Q_SET_DSP0B_MGO

#define HTPX5421Q_DSP1A_RESET_DATA			HTPX5421Q_DSP1A_MRESET_DATA			
#define HTPX5421Q_SET_DSP1A_RESET_ON		HTPX5421Q_SET_DSP1A_MRESET			
#define HTPX5421Q_SET_DSP1A_RESET_OFF		HTPX5421Q_SET_DSP1A_MGO				

#define HTPX5421Q_DSP1B_RESET_DATA			HTPX5421Q_DSP1B_MRESET_DATA			
#define HTPX5421Q_SET_DSP1B_RESET_ON		HTPX5421Q_SET_DSP1B_MRESET			
#define HTPX5421Q_SET_DSP1B_RESET_OFF		HTPX5421Q_SET_DSP1B_MGO				

#define HTPX5421Q_DSP2A_RESET_DATA			HTPX5421Q_DSP2A_MRESET_DATA			
#define HTPX5421Q_SET_DSP2A_RESET_ON		HTPX5421Q_SET_DSP2A_MRESET			
#define HTPX5421Q_SET_DSP2A_RESET_OFF		HTPX5421Q_SET_DSP2A_MGO				

#define HTPX5421Q_DSP2B_RESET_DATA			HTPX5421Q_DSP2B_MRESET_DATA			
#define HTPX5421Q_SET_DSP2B_RESET_ON		HTPX5421Q_SET_DSP2B_MRESET			
#define HTPX5421Q_SET_DSP2B_RESET_OFF		HTPX5421Q_SET_DSP2B_MGO				

#define HTPX5421Q_DSP3A_RESET_DATA			HTPX5421Q_DSP3A_MRESET_DATA			
#define HTPX5421Q_SET_DSP3A_RESET_ON		HTPX5421Q_SET_DSP3A_MRESET			
#define HTPX5421Q_SET_DSP3A_RESET_OFF		HTPX5421Q_SET_DSP3A_MGO				

#define HTPX5421Q_DSP3B_RESET_DATA			HTPX5421Q_DSP3B_MRESET_DATA			
#define HTPX5421Q_SET_DSP3B_RESET_ON		HTPX5421Q_SET_DSP3B_MRESET			
#define HTPX5421Q_SET_DSP3B_RESET_OFF		HTPX5421Q_SET_DSP3B_MGO				

#define HTPX5421Q_SET_DSP_RESET_ON_ALL		HTPX5421Q_SET_DSP_MRESET_ALL		
#define HTPX5421Q_SET_DSP_RESET_OFF_ALL		HTPX5421Q_SET_DSP_MGO_ALL			


	// universal DSP MRESET control macros using DSP core INDEX
#define HTPX5421Q_DSP_MRESET_DATA(n)		(HTPX5421Q_CNTR1_RG_DATA & (1<<(n&0x7))
#define HTPX5421Q_SET_DSP_MRESET(n)			HTPX5421Q_SET_CNTR1_RG(HTPX5421Q_CNTR1_RG_DATA & ~(1<<(n&0x7)))
#define HTPX5421Q_SET_DSP_MGO(n)			HTPX5421Q_SET_CNTR1_RG(HTPX5421Q_CNTR1_RG_DATA | (1<<(n&0x7)))
		// uniform aliases
#define HTPX5421Q_DSP_RESET_DATA(n)			HTPX5421Q_DSP_MRESET_DATA(n)	
#define HTPX5421Q_SET_DSP_RESET_ON(n)		HTPX5421Q_SET_DSP_MRESET(n)
#define HTPX5421Q_SET_DSP_RESET_OFF(n)		HTPX5421Q_SET_DSP_MGO(n)			


	//--- CONTROL-2 register
#define HTPX5421Q_CNTR2_RG_DATA				((*(unsigned *)HTPX5421Q_CNTR2_RG) & HTPX5421Q_RG_DATA_BITMASK)
#define HTPX5421Q_SET_CNTR2_RG(d)			(*(unsigned *)HTPX5421Q_CNTR2_RG) = d

#define HTPX5421Q_HPI_RESET_DATA			(HTPX5421Q_CNTR2_RG_DATA & HTPX5421Q_HPI_RESET_BITMASK)
#define HTPX5421Q_SET_HPI_RESET				HTPX5421Q_SET_CNTR2_RG(HTPX5421Q_CNTR2_RG_DATA & ~HTPX5421Q_HPI_RESET_BITMASK)
#define HTPX5421Q_SET_HPI_GO				HTPX5421Q_SET_CNTR2_RG(HTPX5421Q_CNTR2_RG_DATA | HTPX5421Q_HPI_RESET_GO)
		// uniform aliases for HPI RESET control macros
#define HTPX5421Q_SET_HPI_RESET_ON			HTPX5421Q_SET_HPI_RESET
#define HTPX5421Q_SET_HPI_RESET_OFF			HTPX5421Q_SET_HPI_GO

#define HTPX5421Q_HPI_ERR_EN_DATA			(HTPX5421Q_CNTR2_RG_DATA & HTPX5421Q_HPI_ERR_EN_BITMASK)
#define HTPX5421Q_SET_HPI_ERR_EN_OFF		HTPX5421Q_SET_CNTR2_RG(HTPX5421Q_CNTR2_RG_DATA & ~HTPX5421Q_HPI_ERR_EN_BITMASK) 
#define HTPX5421Q_SET_HPI_ERR_EN_ON			HTPX5421Q_SET_CNTR2_RG(HTPX5421Q_CNTR2_RG_DATA | HTPX5421Q_HPI_ERR_EN_ON)


	//--- HINT STATUS register
#define HTPX5421Q_HINT_STAT_RG_DATA			((*(unsigned *)HTPX5421Q_HINT_STAT_RG) & HTPX5421Q_RG_DATA_BITMASK)

#define HTPX5421Q_HINT0A_DATA				(HTPX5421Q_HINT_STAT_RG_DATA & HTPX5421Q_HINT0A_BITMASK)
#define HTPX5421Q_HINT0B_DATA				(HTPX5421Q_HINT_STAT_RG_DATA & HTPX5421Q_HINT0B_BITMASK)
#define HTPX5421Q_HINT1A_DATA				(HTPX5421Q_HINT_STAT_RG_DATA & HTPX5421Q_HINT1A_BITMASK)
#define HTPX5421Q_HINT1B_DATA				(HTPX5421Q_HINT_STAT_RG_DATA & HTPX5421Q_HINT1B_BITMASK)
#define HTPX5421Q_HINT2A_DATA				(HTPX5421Q_HINT_STAT_RG_DATA & HTPX5421Q_HINT2A_BITMASK)
#define HTPX5421Q_HINT2B_DATA				(HTPX5421Q_HINT_STAT_RG_DATA & HTPX5421Q_HINT2B_BITMASK)
#define HTPX5421Q_HINT3A_DATA				(HTPX5421Q_HINT_STAT_RG_DATA & HTPX5421Q_HINT3A_BITMASK)
#define HTPX5421Q_HINT3B_DATA				(HTPX5421Q_HINT_STAT_RG_DATA & HTPX5421Q_HINT3B_BITMASK)

		// universal HINT status macros using HINT INDEX (HTPX5421Q_HINT0A, etc...)
#define HTPX5421Q_HINT_DATA(n)				(HTPX5421Q_HINT_STAT_RG_DATA & (1<<(n&0x7)))


	//--- HPI Timeout Error STATUS register
#define HTPX5421Q_HPI_ERR_STAT_RG_DATA		((*(unsigned *)HTPX5421Q_HPI_ERR_STAT_RG) & HTPX5421Q_RG_DATA_BITMASK)

#define HTPX5421Q_HPI0_ERR_DATA				(HTPX5421Q_HPI_ERR_STAT_RG_DATA & HTPX5421Q_HPI0_ERR_BITMASK)
#define HTPX5421Q_HPI1_ERR_DATA				(HTPX5421Q_HPI_ERR_STAT_RG_DATA & HTPX5421Q_HPI1_ERR_BITMASK)
#define HTPX5421Q_HPI2_ERR_DATA				(HTPX5421Q_HPI_ERR_STAT_RG_DATA & HTPX5421Q_HPI2_ERR_BITMASK)
#define HTPX5421Q_HPI3_ERR_DATA				(HTPX5421Q_HPI_ERR_STAT_RG_DATA & HTPX5421Q_HPI3_ERR_BITMASK)

		// universal HPI error status macros using HINT INDEX (HTPX5421Q_HPI0_ERR, etc...)
#define HTPX5421Q_HPI_ERR_DATA(n)			(HTPX5421Q_HPI_ERR_STAT_RG_DATA & (1<<(n&0x3)))

	//--- CLEAR HPI ERROR rgisters
#define HTPX5421Q_CLR_HPI0_ERR				(*(unsigned *)HTPX5421Q_CLR_HPI0_ERR_RG) = 0
#define HTPX5421Q_CLR_HPI1_ERR				(*(unsigned *)HTPX5421Q_CLR_HPI1_ERR_RG) = 0
#define HTPX5421Q_CLR_HPI2_ERR				(*(unsigned *)HTPX5421Q_CLR_HPI2_ERR_RG) = 0
#define HTPX5421Q_CLR_HPI3_ERR				(*(unsigned *)HTPX5421Q_CLR_HPI3_ERR_RG) = 0
#define HTPX5421Q_CLR_HPI_ERR_ALL			{HTPX5421Q_CLR_HPI0_ERR;HTPX5421Q_CLR_HPI1_ERR;HTPX5421Q_CLR_HPI2_ERR;HTPX5421Q_CLR_HPI3_ERR;}

		// universal clear HPI error macros using HINT INDEX (HTPX5421Q_HPI0_ERR, etc...)
#define HTPX5421Q_CLR_HPI_ERR(n)			(*(unsigned *)(HTPX5421Q_CLR_HPI0_ERR_RG+((n&0x3)*_WAS_))) = 0 


	//-- PIOX HIRQ Line Enable Register
#define HTPX5421Q_HIRQ_EN_RG_DATA			((*(unsigned *)HTPX5421Q_HIRQ_EN_RG) & HTPX5421Q_RG_DATA_BITMASK)
#define HTPX5421Q_SET_HIRQ_EN_RG(d)			(*(unsigned *)HTPX5421Q_HIRQ_EN_RG) = d

#define HTPX5421Q_HIRQ0_EN_DATA				(HTPX5421Q_HIRQ_EN_RG_DATA & HTPX5421Q_HIRQ0_EN_BITMASK)
#define HTPX5421Q_SET_HIRQ0_EN_OFF			HTPX5421Q_SET_HIRQ_EN_RG(HTPX5421Q_HIRQ_EN_RG_DATA & ~HTPX5421Q_HIRQ0_EN_BITMASK)
#define HTPX5421Q_SET_HIRQ0_EN_ON			HTPX5421Q_SET_HIRQ_EN_RG(HTPX5421Q_HIRQ_EN_RG_DATA | HTPX5421Q_HIRQ0_EN_ON)

#define HTPX5421Q_HIRQ1_EN_DATA				(HTPX5421Q_HIRQ_EN_RG_DATA & HTPX5421Q_HIRQ1_EN_BITMASK)
#define HTPX5421Q_SET_HIRQ1_EN_OFF			HTPX5421Q_SET_HIRQ_EN_RG(HTPX5421Q_HIRQ_EN_RG_DATA & ~HTPX5421Q_HIRQ1_EN_BITMASK)
#define HTPX5421Q_SET_HIRQ1_EN_ON			HTPX5421Q_SET_HIRQ_EN_RG(HTPX5421Q_HIRQ_EN_RG_DATA | HTPX5421Q_HIRQ1_EN_ON)

#define HTPX5421Q_HIRQ2_EN_DATA				(HTPX5421Q_HIRQ_EN_RG_DATA & HTPX5421Q_HIRQ2_EN_BITMASK)
#define HTPX5421Q_SET_HIRQ2_EN_OFF			HTPX5421Q_SET_HIRQ_EN_RG(HTPX5421Q_HIRQ_EN_RG_DATA & ~HTPX5421Q_HIRQ2_EN_BITMASK)
#define HTPX5421Q_SET_HIRQ2_EN_ON			HTPX5421Q_SET_HIRQ_EN_RG(HTPX5421Q_HIRQ_EN_RG_DATA | HTPX5421Q_HIRQ2_EN_ON)

#define HTPX5421Q_HIRQ3_EN_DATA				(HTPX5421Q_HIRQ_EN_RG_DATA & HTPX5421Q_HIRQ3_EN_BITMASK)
#define HTPX5421Q_SET_HIRQ3_EN_OFF			HTPX5421Q_SET_HIRQ_EN_RG(HTPX5421Q_HIRQ_EN_RG_DATA & ~HTPX5421Q_HIRQ3_EN_BITMASK)
#define HTPX5421Q_SET_HIRQ3_EN_ON			HTPX5421Q_SET_HIRQ_EN_RG(HTPX5421Q_HIRQ_EN_RG_DATA | HTPX5421Q_HIRQ3_EN_ON)

#define HTPX5421Q_SET_HIRQ_EN_OFF_ALL		HTPX5421Q_SET_HIRQ_EN_RG(HTPX5421Q_HIRQ_EN_OFF_ALL)
#define HTPX5421Q_SET_HIRQ3_EN_ON_ALL		HTPX5421Q_SET_HIRQ_EN_RG(HTPX5421Q_HIRQ_EN_ON_ALL)


		// universal HIRQ EN access macros using HIRQ INDEX
#define HTPX5421Q_HIRQ_EN_DATA(n)			(HTPX5421Q_HIRQ_EN_RG_DATA & (1<<(n&3)))
#define HTPX5421Q_SET_HIRQ_EN_OFF(n)		HTPX5421Q_SET_HIRQ_EN_RG(HTPX5421Q_HIRQ_EN_RG_DATA & ~(1<<(n&3)))
#define HTPX5421Q_SET_HIRQ_EN_ON(n)			HTPX5421Q_SET_HIRQ_EN_RG(HTPX5421Q_HIRQ_EN_RG_DATA | (1<<(n&3)))


	//-- HIRQ0 IE1/2 Registers
#define HTPX5421Q_HIRQ0_IE1_RG_DATA			((*(unsigned *)HTPX5421Q_HIRQ0_IE1_RG) & HTPX5421Q_RG_DATA_BITMASK)
#define HTPX5421Q_SET_HIRQ0_IE1_RG(d)		(*(unsigned *)HTPX5421Q_HIRQ0_IE1_RG) = d

#define HTPX5421Q_HIRQ0_IE2_RG_DATA			((*(unsigned *)HTPX5421Q_HIRQ0_IE2_RG) & HTPX5421Q_RG_DATA_BITMASK)
#define HTPX5421Q_SET_HIRQ0_IE2_RG(d)		(*(unsigned *)HTPX5421Q_HIRQ0_IE2_RG) = d


#define HTPX5421Q_HIRQ0_HINT0A_IE_DATA		(HTPX5421Q_HIRQ0_IE1_RG_DATA & HTPX5421Q_HINT0A_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ0_HINT0A_IE_OFF	HTPX5421Q_SET_HIRQ0_IE1_RG(HTPX5421Q_HIRQ0_IE1_RG_DATA & ~HTPX5421Q_HINT0A_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ0_HINT0A_IE_ON	HTPX5421Q_SET_HIRQ0_IE1_RG(HTPX5421Q_HIRQ0_IE1_RG_DATA | HTPX5421Q_HINT0A_IE_ON)

#define HTPX5421Q_HIRQ0_HINT0B_IE_DATA		(HTPX5421Q_HIRQ0_IE1_RG_DATA & HTPX5421Q_HINT0B_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ0_HINT0B_IE_OFF	HTPX5421Q_SET_HIRQ0_IE1_RG(HTPX5421Q_HIRQ0_IE1_RG_DATA & ~HTPX5421Q_HINT0B_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ0_HINT0B_IE_ON	HTPX5421Q_SET_HIRQ0_IE1_RG(HTPX5421Q_HIRQ0_IE1_RG_DATA | HTPX5421Q_HINT0B_IE_ON)

#define HTPX5421Q_HIRQ0_HINT1A_IE_DATA		(HTPX5421Q_HIRQ0_IE1_RG_DATA & HTPX5421Q_HINT1A_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ0_HINT1A_IE_OFF	HTPX5421Q_SET_HIRQ0_IE1_RG(HTPX5421Q_HIRQ0_IE1_RG_DATA & ~HTPX5421Q_HINT1A_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ0_HINT1A_IE_ON	HTPX5421Q_SET_HIRQ0_IE1_RG(HTPX5421Q_HIRQ0_IE1_RG_DATA | HTPX5421Q_HINT1A_IE_ON)

#define HTPX5421Q_HIRQ0_HINT1B_IE_DATA		(HTPX5421Q_HIRQ0_IE1_RG_DATA & HTPX5421Q_HINT1B_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ0_HINT1B_IE_OFF	HTPX5421Q_SET_HIRQ0_IE1_RG(HTPX5421Q_HIRQ0_IE1_RG_DATA & ~HTPX5421Q_HINT1B_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ0_HINT1B_IE_ON	HTPX5421Q_SET_HIRQ0_IE1_RG(HTPX5421Q_HIRQ0_IE1_RG_DATA | HTPX5421Q_HINT1B_IE_ON)

#define HTPX5421Q_HIRQ0_HINT2A_IE_DATA		(HTPX5421Q_HIRQ0_IE1_RG_DATA & HTPX5421Q_HINT2A_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ0_HINT2A_IE_OFF	HTPX5421Q_SET_HIRQ0_IE1_RG(HTPX5421Q_HIRQ0_IE1_RG_DATA & ~HTPX5421Q_HINT2A_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ0_HINT2A_IE_ON	HTPX5421Q_SET_HIRQ0_IE1_RG(HTPX5421Q_HIRQ0_IE1_RG_DATA | HTPX5421Q_HINT2A_IE_ON)

#define HTPX5421Q_HIRQ0_HINT2B_IE_DATA		(HTPX5421Q_HIRQ0_IE1_RG_DATA & HTPX5421Q_HINT2B_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ0_HINT2B_IE_OFF	HTPX5421Q_SET_HIRQ0_IE1_RG(HTPX5421Q_HIRQ0_IE1_RG_DATA & ~HTPX5421Q_HINT2B_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ0_HINT2B_IE_ON	HTPX5421Q_SET_HIRQ0_IE1_RG(HTPX5421Q_HIRQ0_IE1_RG_DATA | HTPX5421Q_HINT2B_IE_ON)

#define HTPX5421Q_HIRQ0_HINT3A_IE_DATA		(HTPX5421Q_HIRQ0_IE1_RG_DATA & HTPX5421Q_HINT3A_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ0_HINT3A_IE_OFF	HTPX5421Q_SET_HIRQ0_IE1_RG(HTPX5421Q_HIRQ0_IE1_RG_DATA & ~HTPX5421Q_HINT3A_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ0_HINT3A_IE_ON	HTPX5421Q_SET_HIRQ0_IE1_RG(HTPX5421Q_HIRQ0_IE1_RG_DATA | HTPX5421Q_HINT3A_IE_ON)

#define HTPX5421Q_HIRQ0_HINT3B_IE_DATA		(HTPX5421Q_HIRQ0_IE1_RG_DATA & HTPX5421Q_HINT3B_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ0_HINT3B_IE_OFF	HTPX5421Q_SET_HIRQ0_IE1_RG(HTPX5421Q_HIRQ0_IE1_RG_DATA & ~HTPX5421Q_HINT3B_IE_BITMASK) 
#define HTPX5421Q_SET_HIRQ0_HINT3B_IE_ON	HTPX5421Q_SET_HIRQ0_IE1_RG(HTPX5421Q_HIRQ0_IE1_RG_DATA | HTPX5421Q_HINT3B_IE_ON)

#define HTPX5421Q_HIRQ0_HPI_ERR_IE_DATA		(HTPX5421Q_HIRQ0_IE2_RG_DATA & HTPX5421Q_HPI_ERR_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ0_HPI_ERR_IE_OFF	HTPX5421Q_SET_HIRQ0_IE2_RG(HTPX5421Q_HIRQ0_IE2_RG_DATA & ~HTPX5421Q_HPI_ERR_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ0_HPI_ERR_IE_ON	HTPX5421Q_SET_HIRQ0_IE2_RG(HTPX5421Q_HIRQ0_IE2_RG_DATA | HTPX5421Q_HPI_ERR_IE_ON)

#define HTPX5421Q_SET_HIRQ0_HINT_IE_OFF_ALL	HTPX5421Q_SET_HIRQ0_IE1_RG(HTPX5421Q_HINT_IE_OFF_ALL)
#define HTPX5421Q_SET_HIRQ0_HINT_IE_ON_ALL	HTPX5421Q_SET_HIRQ0_IE1_RG(HTPX5421Q_HINT_IE_ON_ALL)

	//-- HIRQ1 IE1/2 Registers
#define HTPX5421Q_HIRQ1_IE1_RG_DATA			((*(unsigned *)HTPX5421Q_HIRQ1_IE1_RG) & HTPX5421Q_RG_DATA_BITMASK)
#define HTPX5421Q_SET_HIRQ1_IE1_RG(d)		(*(unsigned *)HTPX5421Q_HIRQ1_IE1_RG) = d

#define HTPX5421Q_HIRQ1_IE2_RG_DATA			((*(unsigned *)HTPX5421Q_HIRQ1_IE2_RG) & HTPX5421Q_RG_DATA_BITMASK)
#define HTPX5421Q_SET_HIRQ1_IE2_RG(d)		(*(unsigned *)HTPX5421Q_HIRQ1_IE2_RG) = d


#define HTPX5421Q_HIRQ1_HINT0A_IE_DATA		(HTPX5421Q_HIRQ1_IE1_RG_DATA & HTPX5421Q_HINT0A_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ1_HINT0A_IE_OFF	HTPX5421Q_SET_HIRQ1_IE1_RG(HTPX5421Q_HIRQ1_IE1_RG_DATA & ~HTPX5421Q_HINT0A_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ1_HINT0A_IE_ON	HTPX5421Q_SET_HIRQ1_IE1_RG(HTPX5421Q_HIRQ1_IE1_RG_DATA | HTPX5421Q_HINT0A_IE_ON)

#define HTPX5421Q_HIRQ1_HINT0B_IE_DATA		(HTPX5421Q_HIRQ1_IE1_RG_DATA & HTPX5421Q_HINT0B_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ1_HINT0B_IE_OFF	HTPX5421Q_SET_HIRQ1_IE1_RG(HTPX5421Q_HIRQ1_IE1_RG_DATA & ~HTPX5421Q_HINT0B_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ1_HINT0B_IE_ON	HTPX5421Q_SET_HIRQ1_IE1_RG(HTPX5421Q_HIRQ1_IE1_RG_DATA | HTPX5421Q_HINT0B_IE_ON)

#define HTPX5421Q_HIRQ1_HINT1A_IE_DATA		(HTPX5421Q_HIRQ1_IE1_RG_DATA & HTPX5421Q_HINT1A_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ1_HINT1A_IE_OFF	HTPX5421Q_SET_HIRQ1_IE1_RG(HTPX5421Q_HIRQ1_IE1_RG_DATA & ~HTPX5421Q_HINT1A_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ1_HINT1A_IE_ON	HTPX5421Q_SET_HIRQ1_IE1_RG(HTPX5421Q_HIRQ1_IE1_RG_DATA | HTPX5421Q_HINT1A_IE_ON)

#define HTPX5421Q_HIRQ1_HINT1B_IE_DATA		(HTPX5421Q_HIRQ1_IE1_RG_DATA & HTPX5421Q_HINT1B_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ1_HINT1B_IE_OFF	HTPX5421Q_SET_HIRQ1_IE1_RG(HTPX5421Q_HIRQ1_IE1_RG_DATA & ~HTPX5421Q_HINT1B_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ1_HINT1B_IE_ON	HTPX5421Q_SET_HIRQ1_IE1_RG(HTPX5421Q_HIRQ1_IE1_RG_DATA | HTPX5421Q_HINT1B_IE_ON)

#define HTPX5421Q_HIRQ1_HINT2A_IE_DATA		(HTPX5421Q_HIRQ1_IE1_RG_DATA & HTPX5421Q_HINT2A_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ1_HINT2A_IE_OFF	HTPX5421Q_SET_HIRQ1_IE1_RG(HTPX5421Q_HIRQ1_IE1_RG_DATA & ~HTPX5421Q_HINT2A_IE_BITMASK) 
#define HTPX5421Q_SET_HIRQ1_HINT2A_IE_ON	HTPX5421Q_SET_HIRQ1_IE1_RG(HTPX5421Q_HIRQ1_IE1_RG_DATA | HTPX5421Q_HINT2A_IE_ON)

#define HTPX5421Q_HIRQ1_HINT2B_IE_DATA		(HTPX5421Q_HIRQ1_IE1_RG_DATA & HTPX5421Q_HINT2B_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ1_HINT2B_IE_OFF	HTPX5421Q_SET_HIRQ1_IE1_RG(HTPX5421Q_HIRQ1_IE1_RG_DATA & ~HTPX5421Q_HINT2B_IE_BITMASK) 
#define HTPX5421Q_SET_HIRQ1_HINT2B_IE_ON	HTPX5421Q_SET_HIRQ1_IE1_RG(HTPX5421Q_HIRQ1_IE1_RG_DATA | HTPX5421Q_HINT2B_IE_ON)

#define HTPX5421Q_HIRQ1_HINT3A_IE_DATA		(HTPX5421Q_HIRQ1_IE1_RG_DATA & HTPX5421Q_HINT3A_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ1_HINT3A_IE_OFF	HTPX5421Q_SET_HIRQ1_IE1_RG(HTPX5421Q_HIRQ1_IE1_RG_DATA & ~HTPX5421Q_HINT3A_IE_BITMASK) 
#define HTPX5421Q_SET_HIRQ1_HINT3A_IE_ON	HTPX5421Q_SET_HIRQ1_IE1_RG(HTPX5421Q_HIRQ1_IE1_RG_DATA | HTPX5421Q_HINT3A_IE_ON)

#define HTPX5421Q_HIRQ1_HINT3B_IE_DATA		(HTPX5421Q_HIRQ1_IE1_RG_DATA & HTPX5421Q_HINT3B_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ1_HINT3B_IE_OFF	HTPX5421Q_SET_HIRQ1_IE1_RG(HTPX5421Q_HIRQ1_IE1_RG_DATA & ~HTPX5421Q_HINT3B_IE_BITMASK) 
#define HTPX5421Q_SET_HIRQ1_HINT3B_IE_ON	HTPX5421Q_SET_HIRQ1_IE1_RG(HTPX5421Q_HIRQ1_IE1_RG_DATA | HTPX5421Q_HINT3B_IE_ON)

#define HTPX5421Q_HIRQ1_HPI_ERR_IE_DATA		(HTPX5421Q_HIRQ1_IE2_RG_DATA & HTPX5421Q_HPI_ERR_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ1_HPI_ERR_IE_OFF	HTPX5421Q_SET_HIRQ1_IE2_RG(HTPX5421Q_HIRQ1_IE2_RG_DATA & ~HTPX5421Q_HPI_ERR_IE_BITMASK) 
#define HTPX5421Q_SET_HIRQ1_HPI_ERR_IE_ON	HTPX5421Q_SET_HIRQ1_IE2_RG(HTPX5421Q_HIRQ1_IE2_RG_DATA | HTPX5421Q_HPI_ERR_IE_ON)

#define HTPX5421Q_SET_HIRQ1_HINT_IE_OFF_ALL	HTPX5421Q_SET_HIRQ1_IE1_RG(HTPX5421Q_HINT_IE_OFF_ALL)
#define HTPX5421Q_SET_HIRQ1_HINT_IE_ON_ALL	HTPX5421Q_SET_HIRQ1_IE1_RG(HTPX5421Q_HINT_IE_ON_ALL)

	//-- HIRQ2 IE1/2 Registers
#define HTPX5421Q_HIRQ2_IE1_RG_DATA			((*(unsigned *)HTPX5421Q_HIRQ2_IE1_RG) & HTPX5421Q_RG_DATA_BITMASK)
#define HTPX5421Q_SET_HIRQ2_IE1_RG(d)		(*(unsigned *)HTPX5421Q_HIRQ2_IE1_RG) = d

#define HTPX5421Q_HIRQ2_IE2_RG_DATA			((*(unsigned *)HTPX5421Q_HIRQ2_IE2_RG) & HTPX5421Q_RG_DATA_BITMASK)
#define HTPX5421Q_SET_HIRQ2_IE2_RG(d)		(*(unsigned *)HTPX5421Q_HIRQ2_IE2_RG) = d


#define HTPX5421Q_HIRQ2_HINT0A_IE_DATA		(HTPX5421Q_HIRQ2_IE1_RG_DATA & HTPX5421Q_HINT0A_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ2_HINT0A_IE_OFF	HTPX5421Q_SET_HIRQ2_IE1_RG(HTPX5421Q_HIRQ2_IE1_RG_DATA & ~HTPX5421Q_HINT0A_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ2_HINT0A_IE_ON	HTPX5421Q_SET_HIRQ2_IE1_RG(HTPX5421Q_HIRQ2_IE1_RG_DATA | HTPX5421Q_HINT0A_IE_ON)

#define HTPX5421Q_HIRQ2_HINT0B_IE_DATA		(HTPX5421Q_HIRQ2_IE1_RG_DATA & HTPX5421Q_HINT0B_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ2_HINT0B_IE_OFF	HTPX5421Q_SET_HIRQ2_IE1_RG(HTPX5421Q_HIRQ2_IE1_RG_DATA & ~HTPX5421Q_HINT0B_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ2_HINT0B_IE_ON	HTPX5421Q_SET_HIRQ2_IE1_RG(HTPX5421Q_HIRQ2_IE1_RG_DATA | HTPX5421Q_HINT0B_IE_ON)

#define HTPX5421Q_HIRQ2_HINT1A_IE_DATA		(HTPX5421Q_HIRQ2_IE1_RG_DATA & HTPX5421Q_HINT1A_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ2_HINT1A_IE_OFF	HTPX5421Q_SET_HIRQ2_IE1_RG(HTPX5421Q_HIRQ2_IE1_RG_DATA & ~HTPX5421Q_HINT1A_IE_BITMASK) 
#define HTPX5421Q_SET_HIRQ2_HINT1A_IE_ON	HTPX5421Q_SET_HIRQ2_IE1_RG(HTPX5421Q_HIRQ2_IE1_RG_DATA | HTPX5421Q_HINT1A_IE_ON)

#define HTPX5421Q_HIRQ2_HINT1B_IE_DATA		(HTPX5421Q_HIRQ2_IE1_RG_DATA & HTPX5421Q_HINT1B_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ2_HINT1B_IE_OFF	HTPX5421Q_SET_HIRQ2_IE1_RG(HTPX5421Q_HIRQ2_IE1_RG_DATA & ~HTPX5421Q_HINT1B_IE_BITMASK) 
#define HTPX5421Q_SET_HIRQ2_HINT1B_IE_ON	HTPX5421Q_SET_HIRQ2_IE1_RG(HTPX5421Q_HIRQ2_IE1_RG_DATA | HTPX5421Q_HINT1B_IE_ON)

#define HTPX5421Q_HIRQ2_HINT2A_IE_DATA		(HTPX5421Q_HIRQ2_IE1_RG_DATA & HTPX5421Q_HINT2A_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ2_HINT2A_IE_OFF	HTPX5421Q_SET_HIRQ2_IE1_RG(HTPX5421Q_HIRQ2_IE1_RG_DATA & ~HTPX5421Q_HINT2A_IE_BITMASK) 
#define HTPX5421Q_SET_HIRQ2_HINT2A_IE_ON	HTPX5421Q_SET_HIRQ2_IE1_RG(HTPX5421Q_HIRQ2_IE1_RG_DATA | HTPX5421Q_HINT2A_IE_ON)

#define HTPX5421Q_HIRQ2_HINT2B_IE_DATA		(HTPX5421Q_HIRQ2_IE1_RG_DATA & HTPX5421Q_HINT2B_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ2_HINT2B_IE_OFF	HTPX5421Q_SET_HIRQ2_IE1_RG(HTPX5421Q_HIRQ2_IE1_RG_DATA & ~HTPX5421Q_HINT2B_IE_BITMASK) 
#define HTPX5421Q_SET_HIRQ2_HINT2B_IE_ON	HTPX5421Q_SET_HIRQ2_IE1_RG(HTPX5421Q_HIRQ2_IE1_RG_DATA | HTPX5421Q_HINT2B_IE_ON)

#define HTPX5421Q_HIRQ2_HINT3A_IE_DATA		(HTPX5421Q_HIRQ2_IE1_RG_DATA & HTPX5421Q_HINT3A_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ2_HINT3A_IE_OFF	HTPX5421Q_SET_HIRQ2_IE1_RG(HTPX5421Q_HIRQ2_IE1_RG_DATA & ~HTPX5421Q_HINT3A_IE_BITMASK) 
#define HTPX5421Q_SET_HIRQ2_HINT3A_IE_ON	HTPX5421Q_SET_HIRQ2_IE1_RG(HTPX5421Q_HIRQ2_IE1_RG_DATA | HTPX5421Q_HINT3A_IE_ON)

#define HTPX5421Q_HIRQ2_HINT3B_IE_DATA		(HTPX5421Q_HIRQ2_IE1_RG_DATA & HTPX5421Q_HINT3B_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ2_HINT3B_IE_OFF	HTPX5421Q_SET_HIRQ2_IE1_RG(HTPX5421Q_HIRQ2_IE1_RG_DATA & ~HTPX5421Q_HINT3B_IE_BITMASK) 
#define HTPX5421Q_SET_HIRQ2_HINT3B_IE_ON	HTPX5421Q_SET_HIRQ2_IE1_RG(HTPX5421Q_HIRQ2_IE1_RG_DATA | HTPX5421Q_HINT3B_IE_ON)

#define HTPX5421Q_HIRQ2_HPI_ERR_IE_DATA		(HTPX5421Q_HIRQ2_IE2_RG_DATA & HTPX5421Q_HPI_ERR_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ2_HPI_ERR_IE_OFF	HTPX5421Q_SET_HIRQ2_IE2_RG(HTPX5421Q_HIRQ2_IE2_RG_DATA & ~HTPX5421Q_HPI_ERR_IE_BITMASK) 
#define HTPX5421Q_SET_HIRQ2_HPI_ERR_IE_ON	HTPX5421Q_SET_HIRQ2_IE2_RG(HTPX5421Q_HIRQ2_IE2_RG_DATA | HTPX5421Q_HPI_ERR_IE_ON)

#define HTPX5421Q_SET_HIRQ2_HINT_IE_OFF_ALL	HTPX5421Q_SET_HIRQ2_IE1_RG(HTPX5421Q_HINT_IE_OFF_ALL)
#define HTPX5421Q_SET_HIRQ2_HINT_IE_ON_ALL	HTPX5421Q_SET_HIRQ2_IE1_RG(HTPX5421Q_HINT_IE_ON_ALL)


	//-- HIRQ3 IE1/2 Registers
#define HTPX5421Q_HIRQ3_IE1_RG_DATA			((*(unsigned *)HTPX5421Q_HIRQ3_IE1_RG) & HTPX5421Q_RG_DATA_BITMASK)
#define HTPX5421Q_SET_HIRQ3_IE1_RG(d)		(*(unsigned *)HTPX5421Q_HIRQ3_IE1_RG) = d

#define HTPX5421Q_HIRQ3_IE2_RG_DATA			((*(unsigned *)HTPX5421Q_HIRQ3_IE2_RG) & HTPX5421Q_RG_DATA_BITMASK)
#define HTPX5421Q_SET_HIRQ3_IE2_RG(d)		(*(unsigned *)HTPX5421Q_HIRQ3_IE2_RG) = d


#define HTPX5421Q_HIRQ3_HINT0A_IE_DATA		(HTPX5421Q_HIRQ3_IE1_RG_DATA & HTPX5421Q_HINT0A_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ3_HINT0A_IE_OFF	HTPX5421Q_SET_HIRQ3_IE1_RG(HTPX5421Q_HIRQ3_IE1_RG_DATA & ~HTPX5421Q_HINT0A_IE_BITMASK) 
#define HTPX5421Q_SET_HIRQ3_HINT0A_IE_ON	HTPX5421Q_SET_HIRQ3_IE1_RG(HTPX5421Q_HIRQ3_IE1_RG_DATA  | HTPX5421Q_HINT0A_IE_ON)

#define HTPX5421Q_HIRQ3_HINT0B_IE_DATA		(HTPX5421Q_HIRQ3_IE1_RG_DATA & HTPX5421Q_HINT0B_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ3_HINT0B_IE_OFF	HTPX5421Q_SET_HIRQ3_IE1_RG(HTPX5421Q_HIRQ3_IE1_RG_DATA & ~HTPX5421Q_HINT0B_IE_BITMASK) 
#define HTPX5421Q_SET_HIRQ3_HINT0B_IE_ON	HTPX5421Q_SET_HIRQ3_IE1_RG(HTPX5421Q_HIRQ3_IE1_RG_DATA | HTPX5421Q_HINT0B_IE_ON)

#define HTPX5421Q_HIRQ3_HINT1A_IE_DATA		(HTPX5421Q_HIRQ3_IE1_RG_DATA & HTPX5421Q_HINT1A_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ3_HINT1A_IE_OFF	HTPX5421Q_SET_HIRQ3_IE1_RG(HTPX5421Q_HIRQ3_IE1_RG_DATA & ~HTPX5421Q_HINT1A_IE_BITMASK) 
#define HTPX5421Q_SET_HIRQ3_HINT1A_IE_ON	HTPX5421Q_SET_HIRQ3_IE1_RG(HTPX5421Q_HIRQ3_IE1_RG_DATA | HTPX5421Q_HINT1A_IE_ON)

#define HTPX5421Q_HIRQ3_HINT1B_IE_DATA		(HTPX5421Q_HIRQ3_IE1_RG_DATA & HTPX5421Q_HINT1B_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ3_HINT1B_IE_OFF	HTPX5421Q_SET_HIRQ3_IE1_RG(HTPX5421Q_HIRQ3_IE1_RG_DATA & ~HTPX5421Q_HINT1B_IE_BITMASK) 
#define HTPX5421Q_SET_HIRQ3_HINT1B_IE_ON	HTPX5421Q_SET_HIRQ3_IE1_RG(HTPX5421Q_HIRQ3_IE1_RG_DATA | HTPX5421Q_HINT1B_IE_ON)

#define HTPX5421Q_HIRQ3_HINT2A_IE_DATA		(HTPX5421Q_HIRQ3_IE1_RG_DATA & HTPX5421Q_HINT2A_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ3_HINT2A_IE_OFF	HTPX5421Q_SET_HIRQ3_IE1_RG(HTPX5421Q_HIRQ3_IE1_RG_DATA & ~HTPX5421Q_HINT2A_IE_BITMASK) 
#define HTPX5421Q_SET_HIRQ3_HINT2A_IE_ON	HTPX5421Q_SET_HIRQ3_IE1_RG(HTPX5421Q_HIRQ3_IE1_RG_DATA | HTPX5421Q_HINT2A_IE_ON)

#define HTPX5421Q_HIRQ3_HINT2B_IE_DATA		(HTPX5421Q_HIRQ3_IE1_RG_DATA & HTPX5421Q_HINT2B_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ3_HINT2B_IE_OFF	HTPX5421Q_SET_HIRQ3_IE1_RG(HTPX5421Q_HIRQ3_IE1_RG_DATA & ~HTPX5421Q_HINT2B_IE_BITMASK) 
#define HTPX5421Q_SET_HIRQ3_HINT2B_IE_ON	HTPX5421Q_SET_HIRQ3_IE1_RG(HTPX5421Q_HIRQ3_IE1_RG_DATA | HTPX5421Q_HINT2B_IE_ON)

#define HTPX5421Q_HIRQ3_HINT3A_IE_DATA		(HTPX5421Q_HIRQ3_IE1_RG_DATA & HTPX5421Q_HINT3A_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ3_HINT3A_IE_OFF	HTPX5421Q_SET_HIRQ3_IE1_RG(HTPX5421Q_HIRQ3_IE1_RG_DATA & ~HTPX5421Q_HINT3A_IE_BITMASK) 
#define HTPX5421Q_SET_HIRQ3_HINT3A_IE_ON	HTPX5421Q_SET_HIRQ3_IE1_RG(HTPX5421Q_HIRQ3_IE1_RG_DATA | HTPX5421Q_HINT3A_IE_ON)

#define HTPX5421Q_HIRQ3_HINT3B_IE_DATA		(HTPX5421Q_HIRQ3_IE1_RG_DATA & HTPX5421Q_HINT3B_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ3_HINT3B_IE_OFF	HTPX5421Q_SET_HIRQ3_IE1_RG(HTPX5421Q_HIRQ3_IE1_RG_DATA & ~HTPX5421Q_HINT3B_IE_BITMASK) 
#define HTPX5421Q_SET_HIRQ3_HINT3B_IE_ON	HTPX5421Q_SET_HIRQ3_IE1_RG(HTPX5421Q_HIRQ3_IE1_RG_DATA | HTPX5421Q_HINT3B_IE_ON)

#define HTPX5421Q_HIRQ3_HPI_ERR_IE_DATA		(HTPX5421Q_HIRQ3_IE2_RG_DATA & HTPX5421Q_HPI_ERR_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ3_HPI_ERR_IE_OFF	HTPX5421Q_SET_HIRQ3_IE2_RG(HTPX5421Q_HIRQ3_IE2_RG_DATA & ~HTPX5421Q_HPI_ERR_IE_BITMASK) 
#define HTPX5421Q_SET_HIRQ3_HPI_ERR_IE_ON	HTPX5421Q_SET_HIRQ3_IE2_RG(HTPX5421Q_HIRQ3_IE2_RG_DATA | HTPX5421Q_HPI_ERR_IE_ON)

#define HTPX5421Q_SET_HIRQ3_HINT_IE_OFF_ALL	HTPX5421Q_SET_HIRQ3_IE1_RG(HTPX5421Q_HINT_IE_OFF_ALL)
#define HTPX5421Q_SET_HIRQ3_HINT_IE_ON_ALL	HTPX5421Q_SET_HIRQ3_IE1_RG(HTPX5421Q_HINT_IE_ON_ALL)

//==========================================================================================


//===== universal functions for set/read HIRQx regs using HIRQ INDEX and IE bit INDEX ======
// 'N' parameter defines HIRQ# (HTPX5421Q_HIRQ0,etc...)
// 'M' parameter defines HINT# (HTPX5421Q_HINT0A_IE,etc..., HTPX5421Q_HPI_ERR_IE)

// local functions declarations
unsigned	HTPX5421Q_hirq_ie_data(unsigned n,unsigned m);
void		HTPX5421Q_set_hirq_ie_off(unsigned n,unsigned m);
void		HTPX5421Q_set_hirq_ie_on(unsigned n,unsigned m);


// universal HIRQx_IE register access functions
#define HTPX5421Q_HIRQ_IE1_RG_DATA(n)		((*(unsigned *)(HTPX5421Q_HIRQ0_IE1_RG+((n&0x3)*2*_WAS_))) & HTPX5421Q_RG_DATA_BITMASK)
#define HTPX5421Q_SET_HIRQ_IE1_RG(n,d)		(*(unsigned *)(HTPX5421Q_HIRQ0_IE1_RG+((n&0x3)*2*_WAS_))) = d

#define HTPX5421Q_HIRQ_IE2_RG_DATA(n)		((*(unsigned *)(HTPX5421Q_HIRQ0_IE1_RG+((n&0x3)*2*_WAS_)+_WAS_)) & HTPX5421Q_RG_DATA_BITMASK)
#define HTPX5421Q_SET_HIRQ_IE2_RG(n,d)		(*(unsigned *)(HTPX5421Q_HIRQ0_IE1_RG+((n&0x3)*2*_WAS_)+_WAS_)) = d


// universal functions for accessing HIRQx_IE register bits
#define HTPX5421Q_HIRQ_IE_DATA(n,m)			HTPX5421Q_hirq_ie_data(n,m)
#define HTPX5421Q_SET_HIRQ_IE_OFF(n,m)		HTPX5421Q_set_hirq_ie_off(n,m)
#define HTPX5421Q_SET_HIRQ_IE_ON(n,m)		HTPX5421Q_set_hirq_ie_on(n,m)


// functions for accessing dedicated HIRQx_IE register bits
#define HTPX5421Q_HIRQ_HINT0A_IE_DATA(n)	(HTPX5421Q_HIRQ_IE1_RG_DATA(n) & HTPX5421Q_HINT0A_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ_HINT0A_IE_OFF(n)	HTPX5421Q_SET_HIRQ_IE1_RG(n,HTPX5421Q_HIRQ_IE1_RG_DATA(n) & ~HTPX5421Q_HINT0A_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ_HINT0A_IE_ON(n)	HTPX5421Q_SET_HIRQ_IE1_RG(n,HTPX5421Q_HIRQ_IE1_RG_DATA(n) | HTPX5421Q_HINT0A_IE_ON)

#define HTPX5421Q_HIRQ_HINT0B_IE_DATA(n)	(HTPX5421Q_HIRQ_IE1_RG_DATA(n) & HTPX5421Q_HINT0B_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ_HINT0B_IE_OFF(n)	HTPX5421Q_SET_HIRQ_IE1_RG(n,HTPX5421Q_HIRQ_IE1_RG_DATA(n) & ~HTPX5421Q_HINT0B_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ_HINT0B_IE_ON(n)	HTPX5421Q_SET_HIRQ_IE1_RG(n,HTPX5421Q_HIRQ_IE1_RG_DATA(n) | HTPX5421Q_HINT0B_IE_ON)

#define HTPX5421Q_HIRQ_HINT1A_IE_DATA(n)	(HTPX5421Q_HIRQ_IE1_RG_DATA(n) & HTPX5421Q_HINT1A_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ_HINT1A_IE_OFF(n)	HTPX5421Q_SET_HIRQ_IE1_RG(n,HTPX5421Q_HIRQ_IE1_RG_DATA(n) & ~HTPX5421Q_HINT1A_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ_HINT1A_IE_ON(n)	HTPX5421Q_SET_HIRQ_IE1_RG(n,HTPX5421Q_HIRQ_IE1_RG_DATA(n) | HTPX5421Q_HINT1A_IE_ON)

#define HTPX5421Q_HIRQ_HINT1B_IE_DATA(n)	(HTPX5421Q_HIRQ_IE1_RG_DATA(n) & HTPX5421Q_HINT1B_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ_HINT1B_IE_OFF(n)	HTPX5421Q_SET_HIRQ_IE1_RG(n,HTPX5421Q_HIRQ_IE1_RG_DATA(n) & ~HTPX5421Q_HINT1B_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ_HINT1B_IE_ON(n)	HTPX5421Q_SET_HIRQ_IE1_RG(n,HTPX5421Q_HIRQ_IE1_RG_DATA(n) | HTPX5421Q_HINT1B_IE_ON)

#define HTPX5421Q_HIRQ_HINT2A_IE_DATA(n)	(HTPX5421Q_HIRQ_IE1_RG_DATA(n) & HTPX5421Q_HINT2A_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ_HINT2A_IE_OFF(n)	HTPX5421Q_SET_HIRQ_IE1_RG(n,HTPX5421Q_HIRQ_IE1_RG_DATA(n) & ~HTPX5421Q_HINT2A_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ_HINT2A_IE_ON(n)	HTPX5421Q_SET_HIRQ_IE1_RG(n,HTPX5421Q_HIRQ_IE1_RG_DATA(n) | HTPX5421Q_HINT2A_IE_ON)

#define HTPX5421Q_HIRQ_HINT2B_IE_DATA(n)	(HTPX5421Q_HIRQ_IE1_RG_DATA(n) & HTPX5421Q_HINT2B_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ_HINT2B_IE_OFF(n)	HTPX5421Q_SET_HIRQ_IE1_RG(n,HTPX5421Q_HIRQ_IE1_RG_DATA(n) & ~HTPX5421Q_HINT2B_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ_HINT2B_IE_ON(n)	HTPX5421Q_SET_HIRQ_IE1_RG(n,HTPX5421Q_HIRQ_IE1_RG_DATA(n) | HTPX5421Q_HINT2B_IE_ON)

#define HTPX5421Q_HIRQ_HINT3A_IE_DATA(n)	(HTPX5421Q_HIRQ_IE1_RG_DATA(n) & HTPX5421Q_HINT3A_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ_HINT3A_IE_OFF(n)	HTPX5421Q_SET_HIRQ_IE1_RG(n,HTPX5421Q_HIRQ_IE1_RG_DATA(n) & ~HTPX5421Q_HINT3A_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ_HINT3A_IE_ON(n)	HTPX5421Q_SET_HIRQ_IE1_RG(n,HTPX5421Q_HIRQ_IE1_RG_DATA(n) | HTPX5421Q_HINT3A_IE_ON)

#define HTPX5421Q_HIRQ_HINT3B_IE_DATA(n)	(HTPX5421Q_HIRQ_IE1_RG_DATA(n) & HTPX5421Q_HINT3B_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ_HINT3B_IE_OFF(n)	HTPX5421Q_SET_HIRQ_IE1_RG(n,HTPX5421Q_HIRQ_IE1_RG_DATA(n) & ~HTPX5421Q_HINT3B_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ_HINT3B_IE_ON(n)	HTPX5421Q_SET_HIRQ_IE1_RG(n,HTPX5421Q_HIRQ_IE1_RG_DATA(n) | HTPX5421Q_HINT3B_IE_ON)

#define HTPX5421Q_HIRQ_HPI_ERR_IE_DATA(n)		(HTPX5421Q_HIRQ_IE2_RG_DATA(n) & HTPX5421Q_HPI_ERR_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ_HPI_ERR_IE_OFF(n)	HTPX5421Q_SET_HIRQ_IE2_RG(n,HTPX5421Q_HIRQ_IE2_RG_DATA(n) & ~HTPX5421Q_HPI_ERR_IE_BITMASK)
#define HTPX5421Q_SET_HIRQ_HPI_ERR_IE_ON(n)		HTPX5421Q_SET_HIRQ_IE2_RG(n,HTPX5421Q_HIRQ_IE2_RG_DATA(n) | HTPX5421Q_HPI_ERR_IE_ON)

//==========================================================================================



//========================== universal macros for HPI access ===============================

// local declaration and defs (shall not be used in application code)
#define	_hpi_rg_addr(n)						(HTPX5421Q_HPIC0A_RG+((n&0x7)*4*_WAS_))

void		HTPX5421Q_set_xhpia(unsigned n,unsigned a);
void 		HTPX5421Q_hpi_init(unsigned n);
void		HTPX5421Q_set_hpi_addr(unsigned n,unsigned a);
unsigned	HTPX5421Q_hpi_addr_data(unsigned n);

#define	_HTPX5421Q_HPIC_RG_OFFSET			0
#define	_HTPX5421Q_HPID_AINC_RG_OFFSET		_WAS_
#define	_HTPX5421Q_HPIA_RG_OFFSET			(2*_WAS_)
#define	_HTPX5421Q_HPID_RG_OFFSET			(3*_WAS_)


// universal HPI registers/bits access macros ('N' defines DSP core: HTPX5421Q_DSP0A, etc..)
#define	HTPX5421Q_HPIC_DATA(n)				((*(unsigned *)(_hpi_rg_addr(n)+_HTPX5421Q_HPIC_RG_OFFSET)) & HTPX5421Q_HPIC_RD_BITMASK)
#define	HTPX5421Q_SET_HPIC(n,d)		 		(*(unsigned *)(_hpi_rg_addr(n)+_HTPX5421Q_HPIC_RG_OFFSET)) = d
#define	HTPX5421Q_HPIA_DATA(n)				((*(unsigned *)(_hpi_rg_addr(n)+_HTPX5421Q_HPIA_RG_OFFSET)) & HTPX5421Q_HPI_DATA_BITMASK)
#define	HTPX5421Q_SET_HPIA(n,a) 			(*(unsigned *)(_hpi_rg_addr(n)+_HTPX5421Q_HPIA_RG_OFFSET)) = a
#define	HTPX5421Q_HPID_DATA(n)				((*(unsigned *)(_hpi_rg_addr(n)+_HTPX5421Q_HPID_RG_OFFSET)) & HTPX5421Q_HPI_DATA_BITMASK)
#define	HTPX5421Q_SET_HPID(n,d) 			(*(unsigned *)(_hpi_rg_addr(n)+_HTPX5421Q_HPID_RG_OFFSET)) = d
#define	HTPX5421Q_HPID_AINC_DATA(n)			((*(unsigned *)(_hpi_rg_addr(n)+_HTPX5421Q_HPID_AINC_RG_OFFSET)) & HTPX5421Q_HPI_DATA_BITMASK)
#define	HTPX5421Q_SET_HPID_AINC(n,d) 		(*(unsigned *)(_hpi_rg_addr(n)+_HTPX5421Q_HPID_AINC_RG_OFFSET)) = d

#define HTPX5421Q_HPIC_HINT_DATA(n)			(HTPX5421Q_HPIC_DATA(n) & HTPX5421Q_HPIC_HINT_BITMASK)
#define HTPX5421Q_CLR_HPIC_HINT(n)			HTPX5421Q_SET_HPIC(n,HTPX5421Q_HPIC_WR_HINT_OFF)
#define HTPX5421Q_SET_HPIC_DSPINT(n)		HTPX5421Q_SET_HPIC(n,HTPX5421Q_HPIC_WR_DSPINT_ON)
#define HTPX5421Q_SET_HPIC_FETCH(n)			HTPX5421Q_SET_HPIC(n,HTPX5421Q_HPIC_WR_FETCH_ON)
#define HTPX5421Q_HPIC_HRDY_DATA(n)			(HTPX5421Q_HPIC_DATA(n) & HTPX5421Q_HPIC_HRDY_BITMASK)
#define HTPX5421Q_HPIC_XHPIA_DATA(n)		(HTPX5421Q_HPIC_DATA(n) & HTPX5421Q_HPIC_XHPIA_BITMASK)
#define HTPX5421Q_SET_HPIC_XHPIA(n)			HTPX5421Q_SET_HPIC(n,HTPX5421Q_HPIC_XHPIA_ON)
#define HTPX5421Q_CLR_HPIC_XHPIA(n)			HTPX5421Q_SET_HPIC(n,HTPX5421Q_HPIC_XHPIA_OFF)

#define HTPX5421Q_HPIA_RESET(n)				HTPX5421Q_SET_HPIA(n,0)
#define	HTPX5421Q_SET_XHPIA(n,a)			HTPX5421Q_set_xhpia(n,a)
#define HTPX5421Q_XHPIA_RESET(n)			HTPX5421Q_SET_XHPIA(n,0)

#define	HTPX5421Q_SET_HPI_ADDR(n,a)			HTPX5421Q_set_hpi_addr(n,a)
#define	HTPX5421Q_HPI_ADDR_DATA(n)			HTPX5421Q_hpi_addr_data(n)

#define	HTPX5421Q_HPI_INIT(n)				HTPX5421Q_hpi_init(n)

// HPI read/write functions, which set XHPIA/HPIA regs and read/write HPI data via HPID rg.
// Functions with _AINC suffix can be used in order the succeeding accesses performed via accessing
// HPID_AINC register via simple macros above.
unsigned	HTPX5421Q_rd_hpi_data(unsigned n,unsigned a);
void		HTPX5421Q_wr_hpi_data(unsigned n,unsigned a,unsigned d);
unsigned	HTPX5421Q_rd_hpi_data_ainc(unsigned n,unsigned a);
void		HTPX5421Q_wr_hpi_data_ainc(unsigned n,unsigned a,unsigned d);

// macros to start DSP exectuion during DSPn_RESET=RUN and HPI_RESET transitions fm RESET to RUN.
// Writing to the 0x2F location via HPI makes DSP core to begi execution.
#define		HTPX5421Q_WR_HPI_DSP_START(n)	HTPX5421Q_wr_hpi_data(n,0x2f,0)

//==========================================================================================


//========================== program load functions ========================================
// These functions comprises of functions for loading C5421 DSP code fm host TORNADO DSP.

// This function sets DSPn RESET and loads  'B RESET' code at the DSP RESET vector address.
// After DSP RESET will be released, this will put DSPn to the known state and will allow to
// use JTAG emulator, which requires that all eight DSP cores in JTAG path must be in the
// RUN state.
int		HTPX5421Q_load_default_program(unsigned n);

// This function loads C54xx C-BSF program fm data array to C5421 DSP on-chip environments.
int		HTPX5421Q_load_program(unsigned n, unsigned *p);


//==========================================================================================


//========================== host PIOX-16 I/F init =========================================
// This function performs software reset of host PIOX-16 I/F for TPX5421Q.

int	HTPX5421Q_init();
//==========================================================================================


//============================= error codes ================================================
// error codes are returned by some local functions

#define	HTPX5421Q_OK						0		// no error
#define	HTPX5421Q_PARAM_ERR					-1		// invalid input parameter
#define	HTPX5421Q_HW_ERR					-10		// hardware error
#define	HTPX5421Q_HPI_ERR					-11		// HPI access error
#define	HTPX5421Q_INV_CBSF_DATA_FMT_ERR		-12		// invalid C-BSF data format

//==========================================================================================


/*============================ Local functions =============================================
 Local functions include function code for HPI access, etc. However, in order to  exclude 
 mutiple function body appearance, the local functions shall be excluded from application code 
 for 2nd, 3rd, etc application .OBJ modules. Local functions must be included into the primary
 .OBJ module only. Inclusion/exclusion of local functions is controled via global vars, which 
 can be defined in user code to perferm a particular action:

	__HTPX5421Q_EXCLUDE_AUX_FUNCTIONS__
		- if defined in user code (assigned to any value), then the local 
	      aux functions for HPI access, HPI init, HIF init, etc are excluded 
		  from the user source code. Default is INCLUDED.

	__HTPX5421Q_EXCLUDE_CBSF_PROGRAM_LOADER__
		- if defined in user code (assigned to any value), then the C54x C-BSF loader
		  function (HPTX5421Q_load_program() is excluded from the user source code. 
		  Default is INCLUDED.

*/


#ifndef __HTPX5421Q_EXCLUDE_AUX_FUNCTIONS__
//------------------------- Aux local functions ---------------------------------------------
// Aux local functions include functions for HPI access, HPI init, HIF init, and
// DSP default code loader functions 

//--- HPI access functions

void	HTPX5421Q_set_xhpia(unsigned n,unsigned a)
// set XHPIA register to the <A> data. 
// <A> must be the 16-bit right shifted extended HPI address.
{
	
	HTPX5421Q_SET_HPIC_XHPIA(n);
	HTPX5421Q_SET_HPIA(n,a&0x3);
	HTPX5421Q_CLR_HPIC_XHPIA(n);
}


void	HTPX5421Q_set_hpi_addr(unsigned n,unsigned a)
// set XHPIA/HPIA registers to meet the <A> data. 
{
	
	HTPX5421Q_set_xhpia(n,(a>>16) &0x3);
	HTPX5421Q_SET_HPIA(n,a&0xffff);
}


unsigned	HTPX5421Q_hpi_addr_data(unsigned n)
// returns full XHPIA/HPIA address data for DSP#n. 
{
	unsigned x;
	
	HTPX5421Q_SET_HPIC_XHPIA(n);
	x= ((HTPX5421Q_HPIA_DATA(n) <<16) & 0x30000);
	HTPX5421Q_CLR_HPIC_XHPIA(n);

	return(x | (HTPX5421Q_HPIA_DATA(n) & 0xffff));
}



void 	HTPX5421Q_hpi_init(unsigned n)
// init HPI function
{
	HTPX5421Q_SET_HPIC(n,HTPX5421Q_HPIC_WR_HINT_OFF);
	HTPX5421Q_XHPIA_RESET(n);
	HTPX5421Q_HPIA_RESET(n);
}



void	HTPX5421Q_wr_hpi_data(unsigned n,unsigned a,unsigned d)
// complex function, which set XHPIA/HPIA register to <A> and writes data to via HPID. 
{
	HTPX5421Q_set_xhpia(n,(a>>16) &0x3);
	HTPX5421Q_SET_HPIA(n,a&0xffff);
	HTPX5421Q_SET_HPID(n,d);
}


unsigned	HTPX5421Q_rd_hpi_data(unsigned n,unsigned a)
// complex function, which set XHPIA/HPIA register to <A> and returns HPID data
{
	HTPX5421Q_set_xhpia(n,(a>>16) &0x3);
	HTPX5421Q_SET_HPIA(n,a&0xffff);
	return(HTPX5421Q_HPID_DATA(n));
}

void	HTPX5421Q_wr_hpi_data_ainc(unsigned n,unsigned a,unsigned d)
// complex function, which set XHPIA/HPIA register to <A> and writes data to via HPID_AINC. 
{
	HTPX5421Q_set_xhpia(n,(a>>16) &0x3);
	HTPX5421Q_SET_HPIA(n,a&0xffff);
	HTPX5421Q_SET_HPID_AINC(n,d);
}


unsigned	HTPX5421Q_rd_hpi_data_ainc(unsigned n,unsigned a)
// complex function, which set XHPIA/HPIA register to <A> and returns HPID_AINC data
{
	HTPX5421Q_set_xhpia(n,(a>>16) &0x3);
	HTPX5421Q_SET_HPIA(n,a&0xffff);
	return(HTPX5421Q_HPID_AINC_DATA(n));
}



//----- aux universal HIRQ_IE set/read functions

unsigned	HTPX5421Q_hirq_ie_data(unsigned n,unsigned m)
// universal functions for accessing HIRQx_IE register bits
{
	return((m>=HTPX5421Q_HPI_ERR_IE) ? HTPX5421Q_HIRQ_HPI_ERR_IE_DATA(n):(HTPX5421Q_HIRQ_IE1_RG_DATA(n) & (1<<m)));
}


void		HTPX5421Q_set_hirq_ie_off(unsigned n,unsigned m)
{
	if(m>=HTPX5421Q_HPI_ERR_IE)
			HTPX5421Q_SET_HIRQ_HPI_ERR_IE_OFF(n);
		else
			HTPX5421Q_SET_HIRQ_IE1_RG(n,HTPX5421Q_HIRQ_IE1_RG_DATA(n) & ~(1<<m));
}

void		HTPX5421Q_set_hirq_ie_on(unsigned n,unsigned m)
{
	if(m>=HTPX5421Q_HPI_ERR_IE)
			HTPX5421Q_SET_HIRQ_HPI_ERR_IE_ON(n);
		else
			HTPX5421Q_SET_HIRQ_IE1_RG(n,HTPX5421Q_HIRQ_IE1_RG_DATA(n) | (1<<m));
}




//--- default DSP reset (loop) program loader function

	// C54X <B RESET> command codes and address
#define RESET_LOOP_CMND_DATA_W1	0xf073
#define RESET_LOOP_CMND_DATA_W2	0xff80

#define RESET_LOOP_CMND_ADDR_W1 (HTPX5421Q_HPI_SPDARAM0_SADDR+0xff80-HTPX5421Q_DSP_SPDARAM0_SADDR)

int		HTPX5421Q_load_default_program(unsigned n)
// Set DSPn RESET and load  'B RESET' code at the DSP RESET vector address.
// This function is useful in order to put DSP core into the known state prior releasing
// reset signal for DSP core and starting CCS C5000 JTAG debuger.
{ int r;

	// validate input 'N' parameter
	if(n>HTPX5421Q_DSP3B) return(HTPX5421Q_PARAM_ERR);

	// put DSP to the RESET state
	HTPX5421Q_SET_DSP_MRESET(n);

	// put HPI-RESET to the 'OFF' state in order to enable HPI
	HTPX5421Q_SET_HPI_GO;

	// init HPI port
	HTPX5421Q_HPI_INIT(n);

	// clear HPI error
	HTPX5421Q_CLR_HPI_ERR(n/2);
		
	// load <B RESET> code at the the DSP reset vector (0xff80@P) via HPI port
	HTPX5421Q_wr_hpi_data_ainc(n,RESET_LOOP_CMND_ADDR_W1,RESET_LOOP_CMND_DATA_W1);	
	HTPX5421Q_SET_HPID(n,RESET_LOOP_CMND_DATA_W2);	

	// check for HPI error	
	if(HTPX5421Q_HPI_ERR_DATA(n/2))
	  {
		r=HTPX5421Q_HPI_ERR;
		goto exit;
	  }

	// verify written data
	if(    HTPX5421Q_rd_hpi_data_ainc(n,RESET_LOOP_CMND_ADDR_W1)!=RESET_LOOP_CMND_DATA_W1
		|| HTPX5421Q_HPID_DATA(n)!=RESET_LOOP_CMND_DATA_W2
	  )
	  {
		  r=HTPX5421Q_HW_ERR;
		  goto exit;
	  }

	// check for HPI error again after HPI data verification
	if(HTPX5421Q_HPI_ERR_DATA(n/2))
	  {
	    r=HTPX5421Q_HPI_ERR;
		goto exit;
	  }

	r=HTPX5421Q_OK;	// set no error

 exit:
	// init HPI port in order to set XHPIA=0
	HTPX5421Q_HPI_INIT(n);
	return(r);
}



//--- initialization of all PIOX-16 I/F registers

int	HTPX5421Q_init()
// resets all DSP cores, clears HPI errors, disable host PIOX IRQs
// enable HPI_ERROR control
{	
   HTPX5421Q_SET_CNTR1_RG(0);
   HTPX5421Q_SET_CNTR2_RG(0);
   HTPX5421Q_SET_HPI_ERR_EN_ON;
   HTPX5421Q_CLR_HPI_ERR_ALL;
   HTPX5421Q_SET_HIRQ_EN_OFF_ALL;
   HTPX5421Q_SET_HIRQ0_IE1_RG(0);
   HTPX5421Q_SET_HIRQ0_IE2_RG(0);
   HTPX5421Q_SET_HIRQ1_IE1_RG(0);
   HTPX5421Q_SET_HIRQ1_IE2_RG(0);
   HTPX5421Q_SET_HIRQ2_IE1_RG(0);
   HTPX5421Q_SET_HIRQ2_IE2_RG(0);
   HTPX5421Q_SET_HIRQ3_IE1_RG(0);
   HTPX5421Q_SET_HIRQ3_IE2_RG(0);


   if(	 HTPX5421Q_CNTR1_RG_DATA!=HTPX5421Q_DSP_MRESET_RESET_ALL
	  || HTPX5421Q_CNTR2_RG_DATA!=HTPX5421Q_HPI_ERR_EN_ON
	  || HTPX5421Q_HINT_STAT_RG_DATA!=0
	  || HTPX5421Q_HPI_ERR_STAT_RG_DATA!=0
	  || HTPX5421Q_HIRQ_EN_RG_DATA!=0
	  || HTPX5421Q_HIRQ0_IE1_RG_DATA!=0
	  || HTPX5421Q_HIRQ0_IE2_RG_DATA!=0
	  || HTPX5421Q_HIRQ1_IE1_RG_DATA!=0
	  || HTPX5421Q_HIRQ1_IE2_RG_DATA!=0
	  || HTPX5421Q_HIRQ2_IE1_RG_DATA!=0
	  || HTPX5421Q_HIRQ2_IE2_RG_DATA!=0
	  || HTPX5421Q_HIRQ3_IE1_RG_DATA!=0
	  || HTPX5421Q_HIRQ3_IE2_RG_DATA!=0
	 ) 
		return(HTPX5421Q_HW_ERR);
	
	return(HTPX5421Q_OK);

}
//---------------------------------------------------------------------------------------
#endif	// end of aux local functions code



#ifndef __HTPX5421Q_EXCLUDE_CBSF_PROGRAM_LOADER__
//---------------------- C54x C-BSF target DSP code loader function ---------------------
// C54x C-BSF loader can be used to load C54x DSP code to any DSP core of TPX5421Q DCM.
// C54x C-BSF program data must appear as 32-bit unsigned data array in host TORNADO
// DSP code, which can be converted from source C54x *.OUT file via CBSF54.EXE PC utility.
//
// Notes:
//	1. All C54x DSP on-chip P-MEM and D-MEM target areas can be loaded. 
//	2. I/O areas are invalid and cannot be loaded. 
//	3. In case P-MEM area points to 0x0000..0x7ffff@PROG, then DARAM area is loaded and OVLY 
// 	   bit of PMST is assumed to be set in DSP initialization procedure. 
//	4. Once 0x8000..0xffff@DATA area is addressed, then the corresponding SARAM is assumed 
// 	   and is loaded.


static unsigned __htpx5421q_unpack_cbsf_w16_data(unsigned *i, unsigned **d)
// this function unpacks 16-bit CBSF data from 32-bit data words
// *I++ - W16 counter (even-LSW, odd - MSW, *d++)
// *D   - pointer to 32-bit data words, which increments on odd words
{
	if((((*i)++)%2)==0) 
			return((**d)&0xffff);
	  else
			return(((*((*d)++))>>16)&0xffff);
}


	// C-BSF file defs
#define	_C54X_CBSF_SHD_PMEM_ID		0x9000759e
#define	_C54X_CBSF_SHD_DMEM_ID		0x9001759e
#define	_C54X_CBSF_SHD_IO_ID		0x9002759e
#define	_C54X_CBSF_SHD_TERM_ID		0xffff759e	// last section ID


int		HTPX5421Q_load_program(unsigned n, unsigned *p)
// Set DSP#n RESET and load  program code to the DSP environment.
// C54x program must appear as C-BSF format 32-bit data array at *P data pointer.
{ 
	int r;
	unsigned	shd_id, shd_addr, shd_len;
	unsigned hpi_addr,hpi_addr2, len, len2;
	unsigned *d, i;
  

	// validate input 'N' parameter
	if(		n>HTPX5421Q_DSP3B
	   || 	p==NULL
      ) 
		return(HTPX5421Q_PARAM_ERR);

	// put DSP to the RESET state
	HTPX5421Q_SET_DSP_MRESET(n);

	// put HPI-RESET to the 'GO' state in order to enable HPI
	HTPX5421Q_SET_HPI_GO;

	// init HPI port
	HTPX5421Q_HPI_INIT(n);

	// clear HPI error
	HTPX5421Q_CLR_HPI_ERR(n/2);
		
	// scan over C-BSF section descriptors and load data to DSP on-chip memory
	d = p;
	while(1)
	{
		// fill-in BSF section header
		shd_id 	 = *(d++);
		shd_addr = *(d++);
		shd_len  = *(d++);

		// assume that 2nd HPI area is OFF
		// (for PMEM load with crossing DARAM-SPDARAM boundary)
		hpi_addr2 = 0;
		len2	  = 0;

		// validate SHD params
		r=HTPX5421Q_INV_CBSF_DATA_FMT_ERR;	// assume error

		switch(shd_id)
		{
		  case _C54X_CBSF_SHD_DMEM_ID:
				// section with DMEM data. Verify ADDR/LEN and translate
				// ADDR to HPI address.
				if(	   shd_addr >HTPX5421Q_DSP_SARAM_EADDR
					|| shd_addr <= HTPX5421Q_DSP_MMREG_EADDR
					|| shd_len ==0
					|| shd_len >=HTPX5421Q_DSP_DRAM_PAGE_LEN
					|| (shd_addr+shd_len-1) >HTPX5421Q_DSP_SARAM_EADDR
				  )
					goto exit;
				// translate DMEM address to HPI address for particular DSP core
				hpi_addr = ( (((n%2)==0) ? HTPX5421Q_HPI_DARAMA_BADDR:HTPX5421Q_HPI_DARAMB_BADDR)+shd_addr);
				len = shd_len;
				break;			


		  case _C54X_CBSF_SHD_PMEM_ID:
				// section with PMEM data. Verify ADDR/LEN and translate
				// ADDR to HPI address.
				if(	   shd_addr >HTPX5421Q_DSP_SPDARAM3_EADDR
					|| (shd_addr%HTPX5421Q_DSP_PRAM_PAGE_LEN)<= HTPX5421Q_DSP_MMREG_EADDR
					|| shd_len ==0
					|| shd_len >=HTPX5421Q_DSP_PRAM_PAGE_LEN
					|| (shd_addr+shd_len-1) >HTPX5421Q_DSP_SPDARAM3_EADDR
						// check for inter-page boundary cross
					|| ((shd_addr%HTPX5421Q_DSP_PRAM_PAGE_LEN)+shd_len-1)>=HTPX5421Q_DSP_PRAM_PAGE_LEN
				  )
					goto exit;
					// translate PMEM address to HPI address
					// first check whether this is DARAM with OVLY=1
				if ((shd_addr%HTPX5421Q_DSP_PRAM_PAGE_LEN)<=HTPX5421Q_DSP_DARAM_EADDR)
					{
						// PMEM start address is inside the DARAM/OVLY area
						hpi_addr = ( (((n%2)==0) ? HTPX5421Q_HPI_DARAMA_BADDR:HTPX5421Q_HPI_DARAMB_BADDR)
									 +(shd_addr)%HTPX5421Q_DSP_PRAM_PAGE_LEN);
							
						// now check for crossing DARAM-SPDARAM boundary
						if(((shd_addr+shd_len-1)%HTPX5421Q_DSP_PRAM_PAGE_LEN)>HTPX5421Q_DSP_DARAM_EADDR)
						 {
							// DARAM-SPDARA cross-boundary is identified. 
							// Translate SPDARAM address/len to HPI2 area params.
							len = (HTPX5421Q_DSP_DARAM_EADDR+1)-(shd_addr%HTPX5421Q_DSP_PRAM_PAGE_LEN);
							len2 = shd_len - len;

							switch(shd_addr+len)
							{
								case HTPX5421Q_DSP_SPDARAM0_SADDR:
									hpi_addr2 = HTPX5421Q_HPI_SPDARAM0_SADDR;
									break;

								case HTPX5421Q_DSP_SPDARAM1_SADDR:
									hpi_addr2 = HTPX5421Q_HPI_SPDARAM1_SADDR;
									break;

								case HTPX5421Q_DSP_SPDARAM2_SADDR:
									hpi_addr2 = HTPX5421Q_HPI_SPDARAM2_SADDR;
									break;

								default: // assumed HTPX5421Q_DSP_SPDARAM2_SADDR
									hpi_addr2 = HTPX5421Q_HPI_SPDARAM3_SADDR;
									break;
							}
							break;
						 }
						 else	// no boundary cross
 			 				len = shd_len;
					}
					else
					{	// P-RAM starts and ends within SPDARAM. Translate HPI address.
						switch(shd_addr/HTPX5421Q_DSP_PRAM_PAGE_LEN)
						{
							case 0:
								hpi_addr = (HTPX5421Q_HPI_SPDARAM0_SADDR+shd_addr-HTPX5421Q_DSP_SPDARAM0_SADDR);
								break;

							case 1:
								hpi_addr = (HTPX5421Q_HPI_SPDARAM1_SADDR+shd_addr-HTPX5421Q_DSP_SPDARAM1_SADDR);
								break;

							case 2:
								hpi_addr = (HTPX5421Q_HPI_SPDARAM2_SADDR+shd_addr-HTPX5421Q_DSP_SPDARAM2_SADDR);
								break;

							case 3:
								hpi_addr = (HTPX5421Q_HPI_SPDARAM3_SADDR+shd_addr-HTPX5421Q_DSP_SPDARAM3_SADDR);
								break;
						}
						len = shd_len;
						break;
					}

				break;			


		  case _C54X_CBSF_SHD_IO_ID:
				// loading to I/O area is not supported by PX5421Q
				goto exit;

		  case _C54X_CBSF_SHD_TERM_ID:
				// termination SHD ID. Check for SHD.LEN and SHD.ADDR fields
				// and exit successfully. Otherwise, set error.
				if( shd_addr==0 && shd_len==0)	
					r=HTPX5421Q_OK;	
				goto exit;

				// invalid SHD ID. Exit with error set.
		  default:
				goto exit;
	  	  }
		  
		// now upload data from CBSF array to DSP on-chip memory via HPI port
		// Source 16-bit data is packed into 32-bit data words. Unpack LSW/MSW
		// using word counter. 1st load XHPIA and write data with AINC feature.
		r=HTPX5421Q_HPI_ERR;
		i=0;
		HTPX5421Q_SET_HPI_ADDR(n,hpi_addr);
		for(;len>0;len--)
		{
			// write HPI data using AINC feature
			HTPX5421Q_SET_HPID_AINC(n,__htpx5421q_unpack_cbsf_w16_data(&i,&d));
			// check for HPI error	
			if(HTPX5421Q_HPI_ERR_DATA(n/2))	
				{
					r=HTPX5421Q_HPI_ERR;
					goto exit;
				}
		}				

		//now check whether HPI-2 area must be loaded also in case of boudaries cross
		if(len2!=0)
		{
			HTPX5421Q_SET_HPI_ADDR(n,hpi_addr2);
			for(;len2>0;len2--)
			{
				// write HPI data using AINC feature
				HTPX5421Q_SET_HPID_AINC(n,__htpx5421q_unpack_cbsf_w16_data(&i,&d));
				// check for HPI error	
				if(HTPX5421Q_HPI_ERR_DATA(n/2))
				{
					r=HTPX5421Q_HPI_ERR;
					goto exit;
				}
			}				
		}

		// align W32 ptr: increment W32 ptr if last W16 processed was LSW
		if((i%2)!=0) d++;

	}// end of section scan


 exit:
	// init HPI port in order to set XHPIA=0
	HTPX5421Q_HPI_INIT(n);
	return(r);
}
//---------------------------------------------------------------------------------

#endif	// end of C54x C-BSF loader
//========================================================================================




#endif	// end of TPX5421Q defs

